# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Objets                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

from pylib import *
from mathlib import *

from objet import *


def _alias(obj, unite):
    "Gestion des alias."
    if unite == u"" or unite.startswith("d"):
        return "d"
    if unite.startswith("g"):
        return "g"
    return "r"



class Angle_generique(Objet_numerique):
    u"""Un angle gnrique.

    Classe mre des angles orients ou non. L'unit par dfaut est le radian."""

    unite = __unite = Argument("basestring", None, _alias)
    
    _prefixe_nom = "a"
    
    def __init__(self, unite = "r", **styles):
        self.__unite = unite = Ref(unite)
        Objet.__init__(self, **styles)

    @property
    def radian(self):
        return self.val*self._conversion[self.unite]["r"]
    rad = radian
    
    @property
    def degre(self):
        return self.val*self._conversion[self.unite]["d"]
    deg = degre
    
    @property
    def grad(self):
        return self.val*self._conversion[self.unite]["g"]

    def sin(self):
        return math.sin(self.radian)

    def cos(self):
        return math.cos(self.radian)

    def tan(self):
        return math.tan(self.radian)


##    def _changer_unite(self, unite = None):
##        u"Change l'unit de l'angle en effectuant les adaptations ncessaires."
##        if unite is not None:
##            self._recalculer = True
##            for objet in self.heritiers():
##                objet._recalculer = True
##                if objet.etiquette:
##                    objet.etiquette._recalculer = True
##            self.__unite = self._choix_unite(unite)
##        else:
##            return self.__unite

##    unite = property(_changer_unite, _changer_unite)


    _conversion = {  "r": {"r":1., "d":180./math.pi, "g":200./math.pi},
                    "g": {"r":math.pi/200., "d":180./200., "g":1.},
                    "d": {"r":math.pi/180., "d":1., "g":200./180.}}
    # utilisation: t*conversation["r"]["d"] pour convertir t de radian en degr.

    @staticmethod
    def convertir(objet):
        if isinstance(objet, ALL.Variable):
            return Angle_libre(objet)
        raise TypeError, "%s must be of type 'Variable'" %objet

##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%##
######################### REPRENDRE ICI (FIN DE TRAVAUX) ############################
##%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%##





class Secteur_angulaire(Angle_generique):
    u"Classe mre de tous les angles 'affichables'."

    _style_defaut = param.angles

    point = __point = Argument("Point_generique")
    vecteur1 = __vecteur1 = Argument("Vecteur_generique")
    vecteur2 = __vecteur2 = Argument("Vecteur_generique")
    unite = __unite = Argument("basestring", None, _alias)

    def __init__(self, point, vecteur1, vecteur2, unite = "r", **styles):
        self.__point = point = Ref(point)
        self.__vecteur1 = vecteur1 = Ref(vecteur1)
        self.__vecteur2 = vecteur2 = Ref(vecteur2)
        self.__unite = unite = Ref(unite)
        Angle_generique.__init__(self, unite, **styles)
        self.etiquette = ALL.Label_angle(self)

    def _get_val(self):
        return angle_vectoriel(self.__vecteur1, self.__vecteur2)*self._conversion["r"][self.__unite]

    def _sens(self):
        x1, y1 = self.__vecteur1
        x2, y2 = self.__vecteur2
        return cmp(x1*y2-x2*y1, 0)

    @property
    def sens(self):
        if self._sens() < 0:
            return "indirect"
        return "direct"

    def _affiche(self):
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.fill(), self.__canvas__.plot(), self.__canvas__.plot(), self.__canvas__.plot()]

        for elt_graphique in self.__representation__:
            elt_graphique._visible = True

        x1, y1 = self.__vecteur1.coordonnees
        xx, yy = self.__point.coordonnees
        x2, y2 = self.__vecteur2.coordonnees
        coeff0, coeff1 = self.__canvas__.coeffs()
        u = self.__canvas__.dcoo2pix(x1, y1)
        v = self.__canvas__.dcoo2pix(x2, y2)
        i = (1, 0)
        a = angle_vectoriel(i, u)
        b = angle_vectoriel(i, v)
        if isinstance(self, ALL.Angle) and self._sens() > 0:
            a, b = b, a
        if b<a:
            b += 2*math.pi
        r = param.codage["rayon"]
        x0, y0 = self._pixel(self.__point)
        t = pylab.concatenate((pylab.arange(a, b, self.__canvas__.pas()), [b]))
        niveau = self.style("niveau")


        codage = self.style("codage")
        if not codage:
            for plot in self.__representation__[1:]:
                plot._visible = False
                plot.zorder = niveau
            x, y = self.__canvas__.pix2coo(x0 + r*pylab.cos(t), y0 + r*pylab.sin(t))
        else:
            if codage == "^":
                u = .5*r*pylab.array(u)/norm2(u)
                v = .5*r*pylab.array(v)/norm2(v)
                plot = self.__representation__[1]
                x, y = self.__canvas__.pix2coo( pylab.array((x0 + u[0], x0 + u[0] + v[0], x0 + v[0])), 
                                                                pylab.array((y0 + u[1], y0 + u[1] + v[1], y0 + v[1]))
                                                                )
                plot.set_data(x, y)
                plot._color = self.style("couleur")
                plot._linestyle = self.style("style")
                plot._linewidth = self.style("epaisseur")
                plot._marker = 'None'
                for plot in self.__representation__[2:]:
                    plot._visible = False
                    plot._marker = 'None'
                    plot._color = self.style("couleur")
                    plot._linestyle = self.style("style")
                    plot._linewidth = self.style("epaisseur")

            else:
                n = len(codage)
                for i in xrange(n):
                    plot = self.__representation__[1+i]
                    if i == 0:
                        x, y = self.__canvas__.pix2coo(x0 + (r-3*i)*pylab.cos(t), y0 + (r-3*i)*pylab.sin(t))
                    else:
                        x, y = self.__canvas__.pix2coo(x0 + (r-3*i)*pylab.cos(t), y0 + (r-3*i)*pylab.sin(t))
                    plot.set_data(x, y)
                    plot._color = self.style("couleur")
                    plot._linestyle = self.style("style")
                    plot._linewidth = self.style("epaisseur")
                    plot._marker = 'None'
                for plot in self.__representation__[1+n:]:
                    plot._visible = False
                    plot._marker = 'None'
                    plot._color = self.style("couleur")
                    plot._linestyle = self.style("style")
                    plot._linewidth = self.style("epaisseur")

                if codage in ("/", "x", "o"):
                    c = .5*(a+b)
                    xc, yc = x0 + r*math.cos(c), y0 + r*math.sin(c)
                    taille = param.codage["taille"]
                    if codage == "o":
                        plot = self.__representation__[2]
                        plot._marker = "o"
                        plot._markersize = taille
                        plot._markerfacecolor = "None" # matplotlib 0.91.2
                        plot._markeredgecolor = self.style("couleur")
                        plot._markeredgewidth = self.style("epaisseur")
                        plot._visible = True
                        xo, yo = self.__canvas__.pix2coo(xc, yc)
                        plot.set_data([xo], [yo])
                    else:
                        u = taille*(xc - x0)/r, taille*(yc- y0)/r
                        if codage == "/":
                            plot = self.__representation__[2]
                            xa, ya = self.__canvas__.pix2coo(xc + u[0], yc + u[1])
                            xb, yb = self.__canvas__.pix2coo(xc - u[0], yc - u[1])
                            plot.set_data([xa, xb], [ya, yb])
                            plot._visible = True
                        elif codage == "x":
                            a = param.codage["angle"]
                            plot1 = self.__representation__[2]
                            plot2 = self.__representation__[3]
                            u = pylab.array(u)
                            G = pylab.array((xc, yc))
                            C = G + u
                            M = (pylab.dot([[math.sin(a), -math.cos(a)], [math.cos(a), math.sin(a)]], 
                                    pylab.transpose([C - G])) + pylab.transpose([G]))[:,0]
                            N = (pylab.dot([[math.sin(a), math.cos(a)], [-math.cos(a), math.sin(a)]], 
                                    pylab.transpose([C - G])) + pylab.transpose([G]))[:,0]
                            _x1, _y1 = zip(M, 2*G-M)
                            plot1._x, plot1._y = self.__canvas__.pix2coo(pylab.array(_x1), pylab.array(_y1))
                            plot1._visible = True
                            _x2, _y2 = zip(N, 2*G-N)
                            plot2._x, plot2._y = self.__canvas__.pix2coo(pylab.array(_x2), pylab.array(_y2))
                            plot2._visible = True
                            


        fill = self.__representation__[0]
        fill.xy = zip(x, y) + [(xx, yy)]
        fill._alpha = self.style("alpha")
        fill._facecolor = self.style("couleur")
        fill._linestyle = ALL.FILL_STYLES.get(self.style("style"), 'solid')
        fill._linewidth = self.style("epaisseur")
        fill.zorder = niveau - 0.01



    def _espace_vital(self):
        x, y = self.__point.coordonnees
        x1, y1, = self.__vecteur1.coordonnees
        x2, y2 = self.__vecteur2.coordonnees
        xmin = min(x, x + x1, x + x2)
        xmax = max(x, x + x1, x + x2)
        ymin = min(y, y + y1, y + y2)
        ymax = max(y, y + y1, y + y2)
        return xmin, xmax, ymin, ymax




    def _distance_inf(self, x, y, d):
        x0, y0 = self._pixel(self.__point)
        #print (x - x0)**2 + (y - y0)**2
        if (x - x0)**2 + (y - y0)**2 > (param.codage["rayon"] + d)**2:
            return False
        x, y = self.__canvas__.pix2coo(x, y)
        x0, y0 = self.__point.coordonnees
        u = (x - x0, y - y0)
        if abs(u[0]) + abs(u[1]) < param.tolerance:
            # Vecteur inexistant (le pointeur de la souris est au sommet de l'angle)
            return True
        elif isinstance(self, ALL.Angle) and self._sens() < 0:
            a = angle_vectoriel(self.__vecteur2, self.__vecteur1)
            b = angle_vectoriel(self.__vecteur2, u)
        else:
            a = angle_vectoriel(self.__vecteur1, self.__vecteur2)
            b = angle_vectoriel(self.__vecteur1, u)
        #print a, b
        if a < 0:
            return not a < b < 0
        else:
            return 0 <= b <= a


    def _conditions_existence(self):
        return self.__vecteur1.norme > param.tolerance and self.__vecteur2.norme > param.tolerance






class Angle_oriente(Secteur_angulaire):
    u"""Un angle orient.

    Un angle orient dfini par 3 points A, B, C -> angle (BA>, BC>)."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    unite = __unite = Argument("basestring", None, _alias)

    def __init__(self, point1, point2, point3, unite = "r", **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__unite = unite = Ref(unite)
        Secteur_angulaire.__init__(self, point2, ALL.Vecteur(point2, point1), ALL.Vecteur(point2, point3), unite, **styles)




##    __unite = property(lambda self:self._Angle_generique__unite)




class Angle(Secteur_angulaire):
    u"""Un angle.

    Un angle non orient, dfini par 3 points A, B, C -> angle /ABC\."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")
    unite = __unite = Argument("basestring", None, _alias)

    def __init__(self, point1, point2, point3, unite = "r", **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        self.__unite = unite = Ref(unite)
        Secteur_angulaire.__init__(self, point2, ALL.Vecteur(point2, point1), ALL.Vecteur(point2, point3), unite, **styles)

    def _get_val(self):
        return abs(Secteur_angulaire._get_val(self))


    @property
    def sens(self):
        return u"non dfini" # attention en cas de modification : cette valeur est utilise dans la classe Label_angle





class Angle_libre(Angle_generique):
    u"""Un angle libre.

    Un angle dfini par sa valeur."""

    def _get_valeur(self, value):
        return value*self._conversion["r"][self.__unite]
        
    def _set_valeur(self, value):
        return  value*self._conversion[self.__unite]["r"]
 
    valeur = __valeur = Argument("Variable", _get_valeur, _set_valeur)
    unite = __unite = Argument("basestring", None, _alias)

    def __init__(self, valeur = math.pi/2, unite = "r", **styles):
        self.__unite = unite = Ref(unite)
        self.__valeur = valeur = Ref(valeur)
        Angle_generique.__init__(self, unite)
##        if self.__unite != "r":
##            self.__valeur *= self._conversion[self.__unite]["r"]

    def _affiche(self):
        pass  # les angles libres ne sont pas affichs.

    def _set_val(self, val = None):
#        self._recalculer = True
        self.__valeur = val

    def _get_val(self):
        return self.__valeur

   
#    def _changer_unite(self, unite = "r"):
#        u"Change l'unit de l'angle en effectuant les adaptations ncessaires."
#        unite = self.choix_unite(unite)
#        self.valeur(self.valeur.mult(conversion[self.__unite][unite]))
#        # le code ci-dessus n'est pas parfait. Il marche bien pour les variable contenant simplement une valeur numrique.
#        # mais pour une variable style "A.x", des conversions successives de degr en radian vont faire qqch du style "((((A.x)*0.017453292519943295)*57.295779513082323)*0.017453292519943295)"
#        # En effet, une variable de "type compos" ne sait pas simplifier son code interne pour l'instant.
#        Angle_generique.changer_unite(self, unite)








class Angle_vectoriel(Angle_generique):
    u"""Un angle dfini par des vecteurs.

    Un angle orient dfini par deux vecteurs (non affich)."""
    vecteur1 = __vecteur1 = Argument("Vecteur_generique")
    vecteur2 = __vecteur2 = Argument("Vecteur_generique")
    unite = __unite = Argument("basestring", None, _alias)

    def __init__(self, vecteur1, vecteur2, unite = "r", **styles):
        self.__vecteur1 = vecteur1 = Ref(vecteur1)
        self.__vecteur2 = vecteur2 = Ref(vecteur2)
        self.__unite = unite = Ref(unite)
        Angle_generique.__init__(self, unite)


    def _conditions_existence(self):
        return self.__vecteur1.norme > param.tolerance and self.__vecteur2.norme > param.tolerance


    def _get_val(self):
        return angle_vectoriel(self.__vecteur1, self.__vecteur2)*self._conversion["r"][self.__unite]


del _alias
