# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Objets                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from objet import *







class Interpolation_generique(Objet):
    u"""Classe mre de toute les interpolations."""

    def __init__(self, *points, **styles):
        if len(points) == 1 and isinstance(points[0], (list, tuple)):
            points = tuple(points[0])
        debut = kw.pop("debut", None)
        if debut is None:
            debut = kw.pop("deb", None)
        fin = kw.pop("fin", None)
        Objet.__init__(self)
        self.style(**param.segments)

    def _extremite_arc(self, x, y, vecteur):
        coeff0, coeff1 = self.__canvas__.coeffs()
        if coeff0 == 0 or coeff1 == 0:
            warning("Resolution d'affichage incorrecte (division par zero).")
            return
        vecteur = vecteur[0]/-coeff0, vecteur[1]/coeff1
        if vecteur[0] == 0:
            if vecteur[1] > 0:
                angle = pi/2
            else:
                angle = -pi/2
        else:
            angle = math.arctan(vecteur[1]/vecteur[0])
            if vecteur[0] < 0:
                angle += pi
        # donne l'angle d'incidence a l'extremite
        t = pylab.arange(angle - pi/2, angle + pi/2, 0.05)
        R = self.__canvas__.taille["("]

        x, y = self.__canvas__.coo2pix(x, y)
        return self.__canvas__.pix2coo(x + R*(pylab.cos(t) - cos(angle)), y + R*(pylab.sin(t) - sin(angle)))


    def _affiche_extremites(self, vec_deb = None, vec_fin = None):
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        epaisseur = self.style("epaisseur")
        # Dbut de la courbe
        plot = self.__representation__[-2]
        x, y = self.points[0].coordonnees
        a, b = self.points[1].coordonnees
        if vec_deb is None:
            vec_deb = (x-a, y-b)
        if self.debut == True:
            plot._visible = True
            plot._x, plot._y = pylab.array((x,)), pylab.array((y,))
            plot._color = couleur
            plot._marker = "o"
            plot._markeredgecolor = couleur
            plot._markersize = 2*self.__canvas__.taille["o"]
            plot._linewidth = epaisseur
            plot.zorder = niveau
        elif self.debut == False:
            plot._visible = True
            plot._x, plot._y = self._extremite_arc(x, y, vec_deb)
            plot._color = couleur
            plot._linewidth = epaisseur
            plot.zorder = niveau
        else:
            plot._visible = False

        # Fin de la courbe
        plot = self.__representation__[-1]
        x, y = self.points[-1].coordonnees
        a, b = self.points[-2].coordonnees
        if vec_fin is None:
            vec_fin = (x-a, y-b)
        if self.fin == True:
            plot._visible = True
            plot._x, plot._y = pylab.array((x,)), pylab.array((y,))
            plot._color = couleur
            plot._marker = "o"
            plot._markeredgecolor = couleur
            plot._markersize = 2*self.__canvas__.taille["o"]
            plot._linewidth = epaisseur
            plot.zorder = niveau
        elif self.fin == False:
            plot._visible = True
            plot._x, plot._y = self._extremite_arc(x, y, vec_fin)
            plot._color = couleur
            plot._linewidth = epaisseur
            plot.zorder = niveau
        else:
            plot._visible = False








class Interpolation_lineaire(Interpolation_generique):
    u"""Une interpolation linaire.

    Interpolation entre les points donns par des segments joints (courbe de classe C0).
    """


    def __init__(self, *points, **styles):
        Interpolation_generique.__init__(self, *points, **styles)
        #~ self.points = NotImplemented
        del points

    def _affiche(self):
        n = len(self.points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        if n < 2:
            return

        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x1, y1 = self.points[i].coordonnees
            x2, y2 = self.points[i+1].coordonnees
            plot._x = pylab.array((x1, x2))
            plot._y = pylab.array((y1, y2))
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

        self._affiche_extremites()






class Interpolation_quadratique(Interpolation_generique):
    u"""Une interpolation quadratique.

    Interpolation des points donns par une courbe polynomiale par morceaux, et de classe C1.
    Pour chaque morceau, x(t)=at+bt+c et y(t)=dt+et+f, o t appartient  [0;1].
    """

    def __init__(self, *points, **styles):
        Interpolation_generique.__init__(self, *points, **styles)
        #~ self.points = NotImplemented
        del points


    def _affiche(self):
        n = len(self.points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.points[i].coordonnees
            x1, y1 = self.points[i+1].coordonnees
            if i == 0:
                dx0, dy0 = x1 - x0, y1 - y0
            a, b, c = x1 - dx0 - x0, dx0, x0
            d, e, f = y1 - dy0 - y0, dy0, y0
            plot._x = (a*t + b)*t + c
            plot._y = (d*t + e)*t + f
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

            dx0 = 2*a + b
            dy0 = 2*d + e

        self._affiche_extremites(vec_fin = (dx0, dy0))







class Interpolation_cubique_old(Interpolation_generique):
    u"""Une interpolation cubique.

    Interpolation des points donns par une courbe polynomiale par morceaux, de vecteur tangent horizontal aux sommets, et de classe C1 en gnral (ie. si x_n<>x_{n-1}).
    Pour chaque morceau, x(t)=at^3+bt^2+ct+d et y(t)=et^3+ft^2+gt+h, o t appartient  [0;1], et y'(0)=y'(1)=0.
    """

    def __init__(self, *points, **styles):
        courbure = kw.pop("courbure", 1)
        Interpolation_generique.__init__(self, *points, **styles)
        #~ self.points = NotImplemented
        del points


    def _affiche(self):
        n = len(self.points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.points[i].coordonnees
            x1, y1 = self.points[i+1].coordonnees
            dx0 = dx1 = self.courbure*abs(x1 - x0)
            dy0 = dy1 = 0
            a = -2*(x1 - dx0 - x0)
            b = 3*(x1 - dx0 - x0)
            c = dx0
            d = x0
            e = -2*(y1 - dy0 - y0)
            f = 3*(y1 - dy0 - y0)
            g = dy0
            h = y0
            plot._x = ((a*t + b)*t + c)*t + d
            plot._y = ((e*t + f)*t + g)*t + h
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

        self._affiche_extremites(vec_deb = (dx0, dy0), vec_fin = (dx1, dy1))







class Interpolation_cubique(Interpolation_generique):
    u"""Une interpolation cubique.

    Interpolation des points donns par une courbe polynomiale par morceaux, de vecteur tangent horizontal aux sommets, et de classe C1 en gnral (ie. si x_n<>x_{n-1}).
    Pour chaque morceau, x(t)=at^3+bt^2+ct+d et y(t)=et^3+ft^2+gt+h, o t appartient  [0;1], et y'(0)=y'(1)=0.
    """

    def __init__(self, *points, **styles):
        courbure = kw.pop("courbure", 1)
        Interpolation_generique.__init__(self, *points, **styles)
        #~ self.points = NotImplemented
        del points


    def _affiche(self):
        n = len(self.points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.points[i].coordonnees
            x1, y1 = self.points[i+1].coordonnees
            if i == 0:
                dx0 = x1 - x0
                dy0 = y1 - y0
            else:
                dx0 = self.courbure*abs(x1 - x0)
                if dx1 <> 0:
                    dy0 = dy1/dx1*dx0
                else:
                    dx0 = dx1
                    dy0 = dy1
            if i < n - 2:
                x2, y2 = self.points[i+2].coordonnees
                if cmp(y0, y1) == cmp(y1, y2):
                    dy1 = y1 - y0
                    dx1 = x1 - x0
                else:
                    dy1 = 0
                    dx1 = self.courbure*abs(x1 - x0)
            else:
                dy1 = y1 - y0
                dx1 = x1 - x0
            a = 2*(x0 - x1) + dx0 + dx1
            b = 3*(x1 - x0) -2*dx0 - dx1
            c = dx0
            d = x0
            e = 2*(y0 - y1) + dy0 + dy1
            f = 3*(y1 - y0) -2*dy0 - dy1
            g = dy0
            h = y0
            plot._x = ((a*t + b)*t + c)*t + d
            plot._y = ((e*t + f)*t + g)*t + h
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

        self._affiche_extremites(vec_deb = (dx0, dy0), vec_fin = (dx1, dy1))








class Interpolation_cubique(Interpolation_generique):
    u"""Une interpolation cubique.

    Interpolation des points donns par une courbe polynomiale par morceaux, de vecteur tangent horizontal aux sommets, et de classe C1 en gnral (ie. si x_n<>x_{n-1}).
    Pour chaque morceau, x(t)=at^3+bt^2+ct+d et y(t)=et^3+ft^2+gt+h, o t appartient  [0;1], et y'(0)=y'(1)=0.
    """

    def __init__(self, *points, **styles):
        courbure = kw.pop("courbure", 1)
        Interpolation_generique.__init__(self, *points, **styles)
        #~ self.points = NotImplemented
        del points


    def _affiche(self):
        n = len(self.points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.points[i].coordonnees
            x1, y1 = self.points[i+1].coordonnees
            if i == 0:
                dx0 = x1 - x0
                dy0 = y1 - y0
            else:
                dx0 = self.courbure*abs(x1 - x0)
                if dx1 <> 0:
                    dy0 = dy1/dx1*dx0
                else:
                    dx0 = dx1
                    dy0 = dy1
            if i < n - 2:
                x2, y2 = self.points[i+2].coordonnees
                if cmp(y0, y1) == cmp(y1, y2):
                    dy1 = y2 - y0
                    dx1 = x2 - x0
                else:
                    dy1 = 0
                    dx1 = self.courbure*abs(x1 - x0)
            else:
                dy1 = y1 - y0
                dx1 = x1 - x0
            a = 2*(x0 - x1) + dx0 + dx1
            b = 3*(x1 - x0) -2*dx0 - dx1
            c = dx0
            d = x0
            e = 2*(y0 - y1) + dy0 + dy1
            f = 3*(y1 - y0) -2*dy0 - dy1
            g = dy0
            h = y0
            plot._x = ((a*t + b)*t + c)*t + d
            plot._y = ((e*t + f)*t + g)*t + h
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

        self._affiche_extremites(vec_deb = (dx0, dy0), vec_fin = (dx1, dy1))

#~ class FastPolygone(Polygone):  # DESUET (utiliser le style 'previsualisation' dsormais)
    #~ u"""Un polygone sans ct (usage interne).

    #~ Polygone sans cts destin  un affichage rapide (prvisualisation des constructions)."""

    #~ def __init__(self, *points, **styles):
        #~ if isinstance(points[0], (list, tuple)):
            #~ points = tuple(points[0])
        #~ sommets = points # alias
        #~ centre = centre_gravite = Barycentre(points)
        #~ Objet.__init__(self)
        #~ self.etiquette = Label_polygone(self)
        #~ self.style(**param.polygones)
        #~ self._initialiser([Point_generique])


#~ def f(chaine):
    #~ t = []
    #~ for s in re.findall("[,(][ ]*[A-Za-z0-9]+", chaine):
        #~ nom = s.lstrip(" (,")
        #~ if nom != "self":
            #~ t.append("        self.__"+nom+" = "+nom)
    #~ if t:
        #~ return chaine + '\n' + '\n'.join(t)
    #~ else:
        #~ return chaine

#~ print pylib.regsub('__init__[(]self,[^)]+[)][:]', t, f)

#~ for s in t.split('\nclass '):
    #~ r = re.search('__init__[(]self,[^)]+[)][:]', s)
    #~ if r is not None:
        #~ args = [elt.lstrip(' ,') for elt in re.findall("[,][ ]*[A-Za-z0-9]+", r.group())]
        #~ def f(chaine):
            #~ liste = []
            #~ types = [elt.strip() for elt in chaine[18:-1].split(',')]
            #~ for arg, type in zip(args, types):
                #~ liste.append(arg + " = " + type)
            #~ return chaine[:18] + ", ".join(liste) + chaine[-1:]
        #~ s = pylib.regsub("self[.]_initialiser[(][^)]+[)]", s, f)
    #~ print '\nclass ' + s



















#~ def f(chaine):
    #~ chaine += "\n        self.__args = GestionnaireArguments()"
    #~ t = []
    #~ for s in re.findall("[,(][ ]*[A-Za-z0-9]+", chaine):
        #~ nom = s.lstrip(" (,")
        #~ if nom != "self":
            #~ t.append("        self.__"+nom+" = "+nom+" = Argument("+nom+")")
    #~ if t:
        #~ return chaine + '\n' + '\n'.join(t)
    #~ else:
        #~ return chaine
    
#~ print pylib.regsub('__init__[(]self,[^)]+[)][:]', t, f)
