# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Interpolation                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from objet import *



class Courbe_generique(Objet):
    u"""Classe mre de toutes les courbes."""

    def __init__(self, **styles):
        Objet.__init__(self, **styles)

    def _distance_inf(self, x, y, d):
        if len(self._xarray) and len(self._yarray):
            u, v = self.__canvas__.coo2pix(self._xarray, self._yarray)
            u -= x
            v -= y
            m = min(u*u + v*v) # u*u est beaucoup plus rapide que u**2 (sic!)
            return m < d**2
        return False




class Interpolation_generique(Courbe_generique):
    u"""Classe mre de toutes les interpolations."""

    points = __points = Arguments("Point_generique")
    debut = __fin = Argument("bool")
    fin = __debut = Argument("bool")
    
    _style_defaut = param.segments

    def __init__(self, *points, **styles):
        debut = styles.pop("debut", True)
        fin = styles.pop("fin", True)
        self.__points = points = tuple(Ref(pt) for pt in points)
        self.__debut = debut = Ref(debut)
        self.__fin = fin = Ref(fin)
        Courbe_generique.__init__(self, **styles)
 
##    def _extremite_arc(self, x, y, vecteur):
##        coeff0, coeff1 = self.__canvas__.coeffs()
##        if coeff0 == 0 or coeff1 == 0:
##            warning("Resolution d'affichage incorrecte (division par zero).")
##            return
##        vecteur = vecteur[0]/-coeff0, vecteur[1]/coeff1
##        if vecteur[0] == 0:
##            if vecteur[1] > 0:
##                angle = math.pi/2
##            else:
##                angle = -math.pi/2
##        else:
##            angle = math.arctan(vecteur[1]/vecteur[0])
##            if vecteur[0] < 0:
##                angle += math.pi
##        # donne l'angle d'incidence a l'extremite
##        t = pylab.arange(angle - math.pi/2, angle + math.pi/2, 0.05)
##        R = self.__canvas__.taille["("]
##
##        x, y = self.__canvas__.coo2pix(x, y)
##        return self.__canvas__.pix2coo(x + R*(pylab.cos(t) - math.cos(angle)), y + R*(pylab.sin(t) - math.sin(angle)))


    def _affiche_extremites(self, vec_deb = None, vec_fin = None):
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        epaisseur = self.style("epaisseur")
        # Dbut de la courbe
        plot = self.__representation__[-2]
        x, y = self.__points[0].coordonnees
        a, b = self.__points[1].coordonnees
        if vec_deb is None:
            vec_deb = (x-a, y-b)
        if self.__debut == True:
            plot._visible = True
            plot._x, plot._y = pylab.array((x,)), pylab.array((y,))
            plot._color = couleur
            plot._marker = "o"
            plot._markeredgecolor = couleur
            plot._markersize = 2*self.__canvas__.taille["o"]
            plot._linewidth = epaisseur
            plot.zorder = niveau
        elif self.__debut == False:
            plot._visible = True
            plot._x, plot._y = self._extremite_arc(x, y, vec_deb)
            plot._color = couleur
            plot._linewidth = epaisseur
            plot.zorder = niveau
        else:
            plot._visible = False

        # Fin de la courbe
        plot = self.__representation__[-1]
        x, y = self.__points[-1].coordonnees
        a, b = self.__points[-2].coordonnees
        if vec_fin is None:
            vec_fin = (x-a, y-b)
        if self.__fin == True:
            plot._visible = True
            plot._x, plot._y = pylab.array((x,)), pylab.array((y,))
            plot._color = couleur
            plot._marker = "o"
            plot._markeredgecolor = couleur
            plot._markersize = 2*self.__canvas__.taille["o"]
            plot._linewidth = epaisseur
            plot.zorder = niveau
        elif self.__fin == False:
            plot._visible = True
##            plot._x, plot._y = self._extremite_arc(x, y, vec_fin)
            arc = self.__canvas__.dessine_arc(x, y, vec_fin)
            plot.set_data(arc._x.data, arc._y.data) #  TESTER !!
            plot._color = couleur
            plot._linewidth = epaisseur
            plot.zorder = niveau
        else:
            plot._visible = False








class Interpolation_lineaire(Interpolation_generique):
    u"""Une interpolation linaire.

    Interpolation entre les points donns par des segments joints (courbe de classe C0).
    """

    points = __points = Arguments("Point_generique")
    debut = __fin = Argument("bool")
    fin = __debut = Argument("bool")

    def __init__(self, *points, **styles):
        debut = styles.pop("debut", True)
        fin = styles.pop("fin", True)
        self.__points = points = tuple(Ref(pt) for pt in points)
        self.__debut = debut = Ref(debut)
        self.__fin = fin = Ref(fin)
        Interpolation_generique.__init__(self, *points, **styles)

    def _affiche(self):
        n = len(self.__points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        self._xarray = pylab.array([])
        self._yarray = pylab.array([])
 
        if n < 2:
            return

        pas = self.__canvas__.pas()
       
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x1, y1 = self.__points[i].coordonnees
            x2, y2 = self.__points[i+1].coordonnees
            plot.set_data(pylab.array((x1, x2)), pylab.array((y1, y2)))
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau
            self._xarray = pylab.append(self._xarray, pylab.arange(x1, x2, pas))
            self._xarray = pylab.append(self._xarray, pylab.arange(x1, x2, pas))

        self._affiche_extremites()






class Interpolation_quadratique(Interpolation_generique):
    u"""Une interpolation quadratique.

    Interpolation des points donns par une courbe polynomiale par morceaux, et de classe C1.
    Pour chaque morceau, x(t)=at+bt+c et y(t)=dt+et+f, o t appartient  [0;1].
    """
    points = __points = Arguments("Point_generique")
    debut = __fin = Argument("bool")
    fin = __debut = Argument("bool")

    def __init__(self, *points, **styles):
        debut = styles.pop("debut", True)
        fin = styles.pop("fin", True)
        self.__points = points = tuple(Ref(pt) for pt in points)
        self.__debut = debut = Ref(debut)
        self.__fin = fin = Ref(fin)
        Interpolation_generique.__init__(self, *points, **styles)


    def _affiche(self):
        n = len(self.__points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        self._xarray = pylab.array([])
        self._yarray = pylab.array([])

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.__points[i].coordonnees
            x1, y1 = self.__points[i+1].coordonnees
            if i == 0:
                dx0, dy0 = x1 - x0, y1 - y0
            a, b, c = x1 - dx0 - x0, dx0, x0
            d, e, f = y1 - dy0 - y0, dy0, y0
            u = (a*t + b)*t + c
            v = (d*t + e)*t + f
            plot.set_data(u, v)
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau

            dx0 = 2*a + b
            dy0 = 2*d + e
            
            self._xarray = pylab.append(self._xarray, u)
            self._xarray = pylab.append(self._xarray, v)

        self._affiche_extremites(vec_fin = (dx0, dy0))






class Interpolation_cubique(Interpolation_generique):
    u"""Une interpolation cubique.

    Interpolation des points donns par une courbe polynomiale par morceaux, de vecteur tangent horizontal aux sommets, et de classe C1 en gnral (ie. si x_n<>x_{n-1}).
    Pour chaque morceau, x(t)=at^3+bt^2+ct+d et y(t)=et^3+ft^2+gt+h, o t appartient  [0;1], et y'(0)=y'(1)=0.
    """
    
    points = __points = Arguments("Point_generique")
    debut = __fin = Argument("bool")
    fin = __debut = Argument("bool")
    courbure = __courbure = Argument("Variable")

    def __init__(self, *points, **styles):
        debut = styles.pop("debut", True)
        fin = styles.pop("fin", True)
        courbure = kw.pop("courbure", 1)
        self.__points = points = tuple(Ref(pt) for pt in points)
        self.__debut = debut = Ref(debut)
        self.__fin = fin = Ref(fin)
        self.__courbure = courbure = Ref(fin)
        Interpolation_generique.__init__(self, *points, **styles)
 

    def _affiche(self):
        n = len(self.__points)
        couleur = self.style("couleur")
        niveau = self.style("niveau")
        style = self.style("style")
        epaisseur = self.style("epaisseur")
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot() for i in xrange(n + 1)]

        self._xarray = pylab.array([])
        self._yarray = pylab.array([])

        if n < 2:
            return

        pas = self.__canvas__.pas()
        t = pylab.concatenate((pylab.arange(0, 1, pas), (1,)))
        for i in xrange(n - 1):
            plot = self.__representation__[i]
            x0, y0 = self.__points[i].coordonnees
            x1, y1 = self.__points[i+1].coordonnees
            if i == 0:
                dx0 = x1 - x0
                dy0 = y1 - y0
            else:
                dx0 = self.__courbure*abs(x1 - x0)
                if dx1 <> 0:
                    dy0 = dy1/dx1*dx0
                else:
                    dx0 = dx1
                    dy0 = dy1
            if i < n - 2:
                x2, y2 = self.__points[i+2].coordonnees
                if cmp(y0, y1) == cmp(y1, y2):
                    dy1 = y2 - y0
                    dx1 = x2 - x0
                else:
                    dy1 = 0
                    dx1 = self.__courbure*abs(x1 - x0)
            else:
                dy1 = y1 - y0
                dx1 = x1 - x0
            a = 2*(x0 - x1) + dx0 + dx1
            b = 3*(x1 - x0) -2*dx0 - dx1
            c = dx0
            d = x0
            e = 2*(y0 - y1) + dy0 + dy1
            f = 3*(y1 - y0) -2*dy0 - dy1
            g = dy0
            h = y0
            u = ((a*t + b)*t + c)*t + d
            v = ((e*t + f)*t + g)*t + h
            plot.set_data(u, v)
            plot._color = couleur
            plot._linestyle = style
            plot._linewidth = epaisseur
            plot.zorder = niveau
            
            self._xarray = pylab.append(self._xarray, u)
            self._xarray = pylab.append(self._xarray, v)

        self._affiche_extremites(vec_deb = (dx0, dy0), vec_fin = (dx1, dy1))



class Courbe(Courbe_generique):
    u"""Courbe d'une fonction.
    
    L'expression doit tre donne en fonction de 'x'.
    Exemple : '2x^2-1/x+1'
    """
    
    __re = re.compile("((?<=[^.A-Za-z0-9_])[A-Za-z_][A-Za-z0-9_]*|^[A-Za-z_][A-Za-z0-9_]*)") # correspond  un nom de variable (mais pas d'attribut)

    expression = __expression = Argument("basestring")
    ensemble = __ensemble = Argument("basestring")
    
    def __init__(self, expression, ensemble = "R", **styles):
        self.__expression = expression = Ref(expression)
        self.__ensemble = ensemble = Ref(ensemble)
        Courbe_generique.__init__(self, **styles)
        self.__fonctions = []
        self.__intervalles = []
        
    def _affiche(self):
        if not self.__representation__:
            self.__representation__ = []
        fen = self.__feuille__.fenetre
        for i in xrange(len(self.__fonctions)):
            intervalle = self.__intervalles[i]
            fonction = self.__fonctions[i]
            for array_ in intervalle.asarray(fen[0], fen[1], (fen[1]-fen[0])/param.resolution):
                x = array_
                y = self.__fonction(array_)
                plot = self.__canvas__.plot(x, y)
                plot._color = self.style("couleur")
                plot._linestyle = self.style("style")
                plot._linewidth = self.style("epaisseur")
                plot.zorder = self.style("niveau")
                self.__representation__.append(plot)
#        for extremite in intervalle.extremites(fen[0], fen[1]):
#            valeur, marqueur = extremite
#            if marqueur == ".":
#                
#        plot = self.__representation__[0]
#        u = ((a*t + b)*t + c)*t + d
#        v = ((e*t + f)*t + g)*t + h
#        plot.set_data(u, v)
#        plot._color = self.style("couleur")
#        plot._linestyle = self.style("style")
#        plot._linewidth = self.style("epaisseur")
        
   
    def _test_dependance_circulaire(self, valeur):
        u"""Provoque une erreur si l'objet se retrouve dpendre de lui-mme avec la nouvelle valeur.
        
        Retourne une liste compose alternativement d'instructions et d'objets de la feuille,
        et un ensemble constitu des objets de la feuille mis en jeu dans le code.
        (...  documenter ...)"""
        if isinstance(valeur, basestring) and self.__feuille__ is not None:
            liste = re.split(self.__re, valeur)
            ensemble = set()
            for i in xrange(1, len(liste), 2):
                obj = self.__feuille__.objets[liste[i]]
                if isinstance(obj, Objet):
                    liste[i] = obj
                    ensemble.add(obj)
                    if self is obj or is_in(self, obj._tous_les_ancetres()):
                        print self, 
                        raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(self, obj)
            return liste, ensemble
        return None, None
    
    
    def _compile(self,  liste, ensemble):
        u"""Compile l'expression stocke dans la variable ; les arguments sont les valeurs retournes par '_test_dependance_circulaire'.
        
        La compilation doit toujours avoir lieu  la fin de la procdure de redfinition de la variable,
        car elle ne doit tre excute que si la redfinition de la variable va effectivement avoir lieu,
        c'est--dire si tout le processus prcdent s'est excut sans erreur."""
        if self.__feuille__ is not None:
##            re.findall(self.__re,  self.valeur)
            self.__liste = liste
            self.__fonctions.append(eval("lambda x:" + self.__valeur, self.__feuille__.objets))
#            chaine = "def fonction(x):"
            expressions = self.__expression.split("|")
            ensembles = self.__ensemble.split("|")
            for i in xrange(len(expressions)):
                "if expressions[i]"
            # on supprime la variable de la liste des vassaux pour les objets dont elle ne dpendra plus desormais:
            for objet in self._ancetres: 
                objet.vassaux.remove(self)
            self._ancetres = ensemble
            self._modifier_hierarchie()
            for objet in self._ancetres:   # l'objet est vassal de chacun des objets dont il depend
                objet.vassaux.append(self)
        else:
            for objet in self._ancetres: 
                objet.vassaux.remove(self)
            self._ancetres = set()
            self._modifier_hierarchie()
            self.__liste = []
            self.__fonction = None
    
    def _get_val(self):
        return self.__val_cache # cf. self._conditions_existence
   
    def _set_val(self, valeur = None): 
##        if isinstance(valeur, list): # Rserv  un usage interne (via self.__valeur)
##            raise TypeError,  "l'argument d'une variable doit-etre un nombre ou une chaine de caracteres."
        args = self._test_dependance_circulaire(valeur)
        self.__valeur = valeur
        self._compile(*args)


    def _set_feuille(self):
        self._compile(*self._test_dependance_circulaire(self.__valeur))
        self._heritiers_a_recalculer(self._heritiers())

    @property
    def _type(self):
        return isinstance(self.__valeur, basestring) and "compose" or "simple"


 
    def _recenser_les_ancetres(self):
#        warning("'_recenser_les_ancetres' n'a aucun effet pour une variable.")
        self._modifier_hierarchie()

                
            
    def _conditions_existence(self): # conditions specifiques pour que l'objet existe, a definir pour chaque objet
        if self._type == "compose":
            try:
                self.__val_cache = float(self.__fonction()) # Conversion en float pour l'affichage des points en particulier
            except:
                print_error(u"Impossible de dterminer la valeur de la variable " + self.nom + repr(self))
#                self.__val_cache = None
                return False
        else:
            self.__val_cache = self.valeur
        return True

    
        
    
    def __str__(self):
        return str(self.valeur)
    
    
        

    def varier(self, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        if self.__feuille__ is not None:
            self.__feuille__.start()
            for i in pylab.concatenate((pylab.arange(debut, fin, pas), (fin,))):
                t = time.clock()
                self.val = i
                while time.clock() < t + periode:
                    wx.YieldIfNeeded()
                    if self.__feuille__.__stop__:
                        break
                wx.YieldIfNeeded()
                if self.__feuille__.__stop__:
                    break



    def affiche(self, actualiser = False):
        if actualiser and self.__feuille__ is not None:
            canvas = self.__canvas__
            if canvas:
                canvas.actualiser()
            
        
    def add(self, y):
        u"Addition lie (le rsultat est une variable qui reste toujours gale  la somme des 2 valeurs)."
        if self._soustype() == "simple":
            if isinstance(y, Variable) and y._soustype() == "simple":
                return Variable(self()+y())
            elif isinstance(y, (float, int, long, complex)):
                return Variable(self()+y)
        return Variable("(%s)+(%s)" %(self, y))
    
    def mult(self, y):
        u"Multiplication lie (le rsultat est une variable qui reste toujours gale au produit des 2 valeurs)."
        if self._soustype() == "simple":
            if isinstance(y, Variable) and y._soustype() == "simple":
                return Variable(self()*y())
            elif isinstance(y, (float, int, long, complex)):
                return Variable(self()*y)
        return Variable("(%s)*(%s)" %(self, y))
        
        
        

     
    @staticmethod
    def _convertir(objet):
        u"Convertit un objet en variable."
##        if isinstance(objet, ALL.Variable):   #  Inutile (?)
##            return objet.copy()
        return ALL.Variable(objet)


    def _update(self, objet):
        if not isinstance(objet, Variable):
            objet = self._convertir(objet)
        if isinstance(objet, Variable):
            if objet.__feuille__ is not None:
                objet.__feuille__ = self.__feuille__
            if objet._Variable__fonction is None:
                self.valeur = objet.valeur
            else:
                self.val = objet.val
        else:
            raise TypeError, "l'objet n'est pas une variable."




