# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                  Feuille                                  #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA



# Ce module contient essentiellement la classe Feuille. C'est la classe qui va accueillir tous les objets geometriques.
# Elle fait l'intermediaire entre les objets et le Panel ou s'affichent les objets.

import keyword

from ALL import *
import ALL
import math
import mathlib
import sympy

assert ALL.Objet is Objet
##class Dict_objets(dict):
##    u"""Utilis pour l'instruction exec."""
##    def __init__(self, objets, dico = None):
##        if dico is None:
##            dict.__init__(self)
##        else:
##            dict.__init__(self, dico)
##        self.objets = objets
##        
##    def __setitem__(self, key, val):
##        setattr(self.objets, key, val)
##        
##    def __delitem__(self, key):
##        delattr(self.objets, key)


class Dictionnaire_objets(dict):
    u"""Cette classe est un conteneur pour les objets de la feuille, qui sont tous ses attributs,
sauf ceux prcds de __ (attributs reserves pour tous les objets de la classe).
Elle contient aussi tous les objets geometriques.
Certaines methodes standard (comme __setattr__) sont aussi interceptees ou redefinies.

Attributs spciaux:
'_' fait rfrence au dernier objet enregistr dans la feuille.
"""

    def __init__(self, feuille):
        object.__setattr__(self, "__feuille__", feuille)
        object.__setattr__(self, "__derniere_valeur__", None)
        self.__effacer__()


    def __effacer__(self):
        self.clear()
        # On ajoute au dictionnaire courant les objets gomtriques, et uniquement eux (pas tout LIB.py !)
        self.update((key, val) for key, val in ALL.__dict__.iteritems() if isinstance(val, type) and issubclass(val, ALL.Objet))
        
        # On ajoute au dictionnaire les fonctions et constantes mathmatiques courantes
        self.update((key, val) for key, val in math.__dict__.iteritems() if isinstance(val, (type(eval), float)))
        
        self.update(oo = sympy.oo, Intervalle = intervalles.Intervalle, x = sympy.Symbol("x"), y = sympy.Symbol("y"), z = sympy.Symbol("z"), t = sympy.Symbol("t"))
        
        self.update(pause = self.__feuille__.pause, erreur = self.__feuille__.erreur, effacer = self.__feuille__.effacer, True = True, False = False) # True = True : non compatible Py3k


    __slots__ = ("__feuille__", "__derniere_valeur__")
    def __setattr__(self,nom,val):
        return self.__setitem__(nom, val)
    def __getattr__(self,nom):
        return self.__getitem__(nom)
    def __delattr__(self,nom):
        return self.__delitem__(nom)


    def add(self, valeur):
        self["_"] = valeur
        
    def remove(self, valeur):
        for (key, val) in self.items():
            if val is valeur:
                self.pop(key)
##    def get_(self):     return self.__derniere_valeur__()
##    def set_(self, valeur):     self.add(valeur)
##    def del_(self):     self.__delitem__(self.__derniere_valeur__().nom)
##    _ = property(_get, _set, _del, u"""Fait rfrence au dernier objet enregistr dans la feuille.""")
##    del get_, set_, del_
    
    def __setitem__(self, nom, valeur):
        u"""Note: objets.__setattr__("a", 3) <-> objets.a = 3
        Les objets d'une feuille, contrairement aux objets Python, ne peuvent pas etre redefinis comme ca...
        En general, quand on essaie d'attribuer un nom qui existe deja, ce n'est pas volontaire.
        Pour eviter les erreurs, on impose de detruire explicitement l'objet avant que le nom puisse etre redonne."""
                        
        
        if nom == "fenetre":
            self.__feuille__.fenetre = valeur
            return
        if keyword.iskeyword(nom):
            self.erreur(u"Nom r\xe9serv\xe9 : " + nom, NameError)
            
        nom = self.__verifier_syntaxe_nom__(nom)
        
        # Pour certains types d'objets (les points, textes et variables), le nom peut etre deja attribu.
        # Par exemple, A=Point(1,2) est valide meme si A existe deja.
        # L'objet A ne sera pas ecrase, mais actualise.
        # Dans cet exemple, les coordonnees de A sont actualisees en (1,2)
        # (Autrement dit, A=(1,2) devient un alias de A(1,2) ou A.coordonnees = 1,2).
        # De meme, pour une variable u, u=3 est compris comme u(3) ou u.val = 3, qui est la syntaxe 'orthodoxe'.
        # Bien sur, il faut en particulier que la valeur soit un objet de meme type.
        # (A = Variable(3) n'est pas valide si A est un point !)
##        if self.has_key(nom):
##            if hasattr(self[nom], "_update"):
##                self[nom]._update(valeur)
##                return # on quitte, car le nom doit toujours rfrer  l'objet initial !
##            else:
##                self.erreur(u"ce nom est deja utilis\xe9 : " + nom, NameError)
        if self.has_key(nom):
            try:
                self[nom]._update(valeur)
                return # on quitte, car le nom doit toujours rfrer  l'objet initial !
            except:
                print_error()
                self.erreur(u"Ce nom est deja utilis\xe9 : " + nom, NameError)
          
          

        if not isinstance(valeur, Objet):
        # Par convniance, certains types sont automatiquement convertis : 
            # - Variable
            if isinstance(valeur, (int, long, float, str, unicode)): # u=3 cree une variable
                valeur = Variable(valeur)
                
            # - Point
            elif isinstance(valeur, complex):
                valeur = Point(valeur.real, valeur.imag)
            
            elif hasattr(valeur, "__iter__"):
                valeur = tuple(valeur)
                # - Texte
                if len(valeur) in (1, 3) and isinstance(tuple(valeur)[0], basestring):
                    # t=["Bonjour!"] cree un texte
                    valeur = Texte(*valeur)
                elif len(valeur) == 2:
                    # - Vecteur_libre
                    if nom.islower():
                        # u=(1,2) cre un vecteur
                        valeur = Vecteur_libre(*valeur)
                    # - Point
                    else:
                        # A=(1,2) cree un point.
                        valeur = Point(*valeur) 

        if not isinstance(valeur, Objet):
            self.erreur("type d'objet incorrect :(%s,%s)"%(nom, valeur), TypeError)
                
        #~ if valeur.__feuille__ is not self.__feuille__:
            #~ # Indique  l'objet lors de son initialisation dans quelle feuille il est cr
            #~ warning("L'objet a ete initialise en ayant une autre feuille comme reference.\nObjet.__feuille__ aurait du pointer sur la feuille courante.")
            #~ valeur.__feuille__ = self.__feuille__
        valeur.__feuille__ = self.__feuille__

        if nom == "_":
            nom = valeur._nom_alea() # Attention, la feuille doit etre deja definie !
            # Pour les objets nomms automatiquement, le nom n'est pas affich par dfaut.
            if valeur.style("legende") == wxgeo.NOM:
                valeur.style(legende = wxgeo.RIEN)

        if is_in(valeur, self.itervalues()): # si l'objet est deja reference sur la feuille, on en fait une copie.
            valeur = valeur.copy() # ainsi, A = B est remplace par A = B.copy()
        
        # les objets commencant par "_" ne sont pas affichs par dfaut (pure convention) :
        if nom[0] == "_":
            valeur.style(visible = False)
 
        dict.__setitem__(self, nom, valeur)
        dict.__setattr__(self,  "__derniere_valeur__", weakref.ref(valeur))
        valeur.refresh()


    def __getitem__(self, nom):
        if nom == "fenetre":
            return self.__feuille__.fenetre
        elif nom == "objets":
            return self()
        elif nom == "_":
            return self.__derniere_valeur__()
        try:
            return dict.__getitem__(self, self.__convertir_nom__(nom))
        except KeyError:
            self.erreur(u"Objet introuvable sur la feuille : " + nom, KeyError)
        
        
    def __delitem__(self, nom):
        if nom == "_":
            self.__derniere_valeur__().supprimer()
        self[nom].supprimer()
        
        
    def __call__(self, objets_caches = True, **kw):
        u"""Retourne la liste des objets gomtriques.
        Le paramtre objets_caches indique s'il faut retourner les objets cachs.
        kw:
        'type' : types  inclure
        'sauf' : types  exclure
        """
        # Note: objets.__call__() <-> objets()
        if kw:
            sauf = kw.get("sauf", ())
            type = kw.get("type", Objet)
            return [obj for obj in self.values() if isinstance(obj, type) and not isinstance(obj, sauf) and (objets_caches or obj.style("visible"))]
        elif objets_caches:
            return [obj for obj in self.values() if isinstance(obj, Objet)]
        else:
            return [obj for obj in self.values() if isinstance(obj, Objet) and obj.style("visible")]
            
    
    def __noms__(self, objets_caches = True, **kw):
        u"""Retourne les noms de tous les objets gomtriques.
        Le paramtre objets_caches indique s'il faut retourner les objets cachs.
        kw:
        'type' : types  inclure
        'sauf' : types  exclure
        """
        if kw:
            sauf = kw.get("sauf", ())
            type = kw.get("type", Objet)
            return [nom for nom, obj in self.items() if isinstance(obj, type) and not isinstance(obj, sauf) and (objets_caches or obj.style("visible"))]
        else:
            return [nom for nom, obj in self.items() if isinstance(obj, Objet) and (objets_caches or obj.style("visible"))]
    
    
    
    
    @staticmethod
    def __convertir_nom__(nom, inverse = False):
        u"Convertit les noms termins par des ` en noms python corrects."
        if inverse:
            return nom.replace("_", "`")
        return nom.replace('`', '_').replace('"', '__').replace("'", "_")
        
    def __verifier_syntaxe_nom__(self, nom):
        u"Vrifie que le nom est correct (ie. bien form) et le modifie le cas chant."
        if nom == "":
            return "_"
        if nom.startswith("__"):
            self.erreur(u'un nom ne peut commencer par "__".', SyntaxError)
        if not re.match("""[A-Za-z_][A-Za-z0-9_]*['"]?['"]?[']?$""", nom):
            self.erreur(u"'%s' n'est pas un nom d'objet valide." %nom, SyntaxError)
        return self.__convertir_nom__(nom)
    
    
    def __verifier_nom__(self, nom):
        u"Vrifie que le nom peut-tre attrivu (c--d. qu'il est bien form, et non utilis)."
        self.__verifier_syntaxe_nom__(nom)
        if self.has_key(nom):
            self.erreur(u"ce nom est deja utilis.", NameError)
        return nom

        
    
##    def effacer(self):
##        for key, obj in self.items():
##            if isinstance(obj, Objet):
##                try:
##                    del self[key]
##                except KeyError:
##                    pass # la cl a dj t efface (par exemple, il s'agit d'un segment dont une des extrmits a dj t supprime)

 








class Historique_feuille(object):
    u"""Historique de la feuille.
    Permet d'enregistrer l'tat de la feuille  un instant donn, et de le restaurer ensuite."""
    
    def __init__(self, feuille):
        self.__feuille__ = feuille
        self.n = param.nbr_annulations
        self.etats = []
        #self.etats_annules = []
        #self.derniere_commande = ""
        self.archiver()
        
    def archiver(self, commande = ""):
        # avec la molette de la souris, on effectue une succession rapide de zooms
        # pour eviter que ca ne remplisse l'historique, on archive alors l'etat actuel a la place du precedent.
        if commande.startswith("zoomer(") and self.derniere_commande.startswith("zoomer("):
            self.etats[-1] = self.__feuille__.sauvegarder()#False)
        else:
            self.etats.append(self.__feuille__.sauvegarder())#False))
        self.etats_annules = []
        if len(self.etats) > self.n:
            self.etats.pop(0) # plus rapide que "self.etats = self.etats[-self.n:]"
        self.derniere_commande = commande
    
    def annuler(self):
        if len(self.etats) > 1:
            etat_actuel = self.etats.pop()
            self.etats_annules.append(etat_actuel)
            if len(self.etats_annules) > self.n:
                self.etats_annules.pop(0) # plus rapide que "self.etats_annules = self.etats_annules[-self.n:]"
            self.restaurer(self.etats[-1])
            self.__feuille__.message(u"Action annule.")
        else:
            self.__feuille__.message(u"Impossible d'annuler.")



    def refaire(self):
        if self.etats_annules:
            etat = self.etats_annules.pop()
            self.etats.append(etat)
            if len(self.etats) > self.n:
                self.etats.pop(0) # plus rapide que "self.etats = self.etats[-self.n:]"
            self.restaurer(etat)
            self.__feuille__.message(u"Action restaure.")
        else:
            self.__feuille__.message(u"Impossible de restaurer.")

                
##    def restaurer(self, txt):
##        #~ commandes = regexp(u"set_fenetre[(][-0-9., ]*[)]", txt, u"'__fenetre__=[' + x[12:-1] + ']'").split("\n")
##        commandes = regsub(u"set_fenetre[(][-0-9., ]*[)]", txt, lambda x:'__fenetre__=[' + x[12:-1] + ']').split("\n")
##        #debug(commandes)
##        self.__feuille__.objets.effacer()     
##        exec(commandes[0], self.__feuille__.__dict__)
##        self.__feuille__.canvas.geler_affichage = True
##        try:
##            for commande in commandes[1:]:
##                exec(commande, self.__feuille__.objets)
##            self.__feuille__.canvas.commande.actualiser()
##        finally:
##            self.__feuille__.canvas.geler_affichage = False
##        self.__feuille__.refresh()

    def restaurer(self, txt):
        self.__feuille__.effacer(False)
        self.__feuille__.charger(txt)










class Feuille(object):
    u"Feuille de travail"
    
    def __hash__(self):
        return id(self)
    
    def __init__(self, parent = None, canvas = None, titre = ""):
        self.parent = parent
        self.canvas = None
        if parent:
            if not canvas:
                self.canvas = self.parent.canvas
            self.__fenetre__ = parent.param("fenetre")
        else:
            self.__fenetre__ = param.fenetre
        if canvas:
            self.canvas = canvas
        self.objets = Dictionnaire_objets(self)
        self.historique = Historique_feuille(self)
        self.macros = {}
        self.__objets_temporaires__ = []
        self.__point_temporaire__ = None
        self.action_principale = None # verrou qui indique que des objets sont encore en cours de modification
        self.__stop__ = False # on le met  True pour stopper toutes les animations en cours
        
        # Parametres permettant de gerer l'enregistrement:
        self.sauvegarde = {
                    "modifie": True,            # modifications depuis dernire sauvegarde
                    "repertoire": None,         # rpertoire de sauvegarde
                    "nom": None,          # nom de sauvegarde
                    "export": None,        # nom complet utilis pour l'export
                    }
                    
        # Informations sur le document
        self.infos = {
            "titre": titre,
            "auteur": param.utilisateur,
            "creation": time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime()),
            "modification": time.strftime("%d/%m/%Y - %H:%M:%S",time.localtime()),
            "version": "",
            "resume": "",
            "notes": "",
            }
        
            
#        Objet.__feuille__ = self # les objets sont crees dans cette feuille par defaut
                
        
        


#########################################################################################    

# Les fonctions suivantes permettent de passer facilement de l'objet a son nom, et vice-versa...
# Note : il n'y a pas necessairement bijection entre l'ensemble des objets et celui des noms,
# deux noms pouvant correspondre au meme objet. Dans ce cas, le nom renvoye est un des noms de l'objet.


##    def nom_objet(self, objet, liste = False):
##        # Renvoie le nom de l'objet, ou celui de la feuille en l'absence d'objet.
##        liste_noms = [key for key, val in self.objets.items() if val is objet]
##        if liste: return liste_noms # si liste == True, renvoie la liste des noms.
##        if liste_noms: return liste_noms[0] # sinon, le premier nom.
##        return  ""

    @property
    def nom(self):
        u"Destin  tre affich."
        nom = self.identifiant()
        if not self.infos["titre"] or re.match("Feuille [1-9][0-9]*$", self.infos["titre"]): # le titre est identique  l'identifiant
            if self.sauvegarde["nom"]:
                return self.identifiant() + " - " + self.sauvegarde["nom"] # il y a un nom de sauvegarde
            else:
                return self.identifiant()
        else:   # le titre est diffrent, on l'affiche
            return self.identifiant() + " - " + self.infos["titre"]
        
    def nom_complet(self):
        u"Destin  tre affich en haut de la fentre."
        nom = self.sauvegarde["modifie"] and "* " or ""
        liste = self.sauvegarde["nom"], self.infos["titre"]
        nom += " - ".join(s for s in liste if s)
        return nom
    
    
    def identifiant(self):
        u"""Contrairement au nom, l'identifiant est unique et constant durant toute la session.
        Il est de la forme 'Feuille n', avec n > 0."""
        
        return self.parent.nom_feuille(self)
        
    def id(self):
        u"Numero de la feuille."
        return self.parent.num_feuille(self)
    

    def objet(self, nom):    # retourne l'objet associe au nom "nom"
        return self.objets[nom]
        
    
    
#######################################################################################

# Methodes se rapportant a la feuille elle-meme


    def redefinir(self, objet, valeur):
        nom = objet.nom
        valeur = valeur.strip()
        i = valeur.find("(")
        args = eval(valeur[i:], self.objets) # FAILLE POTENTIELLE DE SECURITE
        heritiers = objet._heritiers() 
        heritiers.add(objet)
        for arg in args:
            if isinstance(arg, Objet):
                for heritier in heritiers:
                    if arg is heritier:
                        self.erreur(u"Dfinition circulaire dans %s : l'objet %s se retrouve dpendre de lui-mme." %(valeur, nom))
                        #raise RuntimeError, "Definition circulaire dans %s : l'objet %s se retrouve dependre de lui-meme." %(valeur, nom)
        actuel = self.sauvegarder()
        if isinstance(objet, Texte):
            nouveau = actuel.replace("\n" + nom + "=" + objet.__repr__(False), "\n" + nom + "=" + valeur)
        else:
            valeur += "\n" + nom + ".copier_style(" + repr(objet) + ")"
            nouveau = actuel.replace("\n" + nom + "=" + repr(objet), "\n" + nom + "=" + valeur)
        print nouveau
        try:
            self.historique.restaurer(nouveau)
        except:
            print_error()
            self.historique.restaurer(actuel)
            self.erreur("Erreur lors de la redfinition de %s." %nom)
        
        
#    def objets_libres(self):
#        return [obj for obj in self.objets() if obj.hierarchie() == 0]
        
#    def nom_objets_libres(self):
#        return [obj.nom() for obj in self.objets() if obj.hierarchie() == 0]
        
        
    def inventaire(self):
        if param.debug:
            for obj in self.objets():   print "- " + obj.nom + " : " + repr(obj) + " (" + obj.type() + ")"
        liste = [unicode2(obj.nom_complet) + u" (" + unicode2(obj.titre(point_final = False)) + (not obj.style("visible") and u" invisible)" or u")") for obj in self.objets()]
        liste.sort()
        return liste
            
    def effacer_codage(self):
        u"Efface tous les codages sur les segments, angles et arcs de cercles."
        for obj in self.objets():
            if obj.style("codage") is not None:
                obj.style(codage = "")
                obj.affiche()
        if self.canvas:
            self.canvas.actualiser()
        
        
    def coder(self):
        u"Codage automatique de la figure (longueurs gales, angles gaux, et angles droits)."
        def test(groupe, liste, i):
            if len(groupe) is 1:
                groupe[0]["objet"].style(codage = "")
                groupe[0]["objet"].affiche()
                return False
            else:
                try:
                    for elt in groupe:
                        elt["objet"].style(codage = liste[i])
                        elt["objet"].affiche()
                    return True
                except IndexError:
                    self.message(u"Le nombre de codages disponibles est insuffisant.")
                    print_error(u"Le nombre de codages disponibles est insuffisant.")

        lignes = [{"longueur": obj._longueur(), "objet": obj} for obj in self.objets(False, type = (Segment, Arc_generique))]
        if lignes:
            lignes.sort() # attention, le classement d'un dictionnaire se fait selon l'ordre alphabtique des clefs
            groupe = [lignes[0]]
            i = 1
            for ligne in lignes[1:]:
                if abs(groupe[-1]["longueur"] - ligne["longueur"]) < param.tolerance:
                    groupe.append(ligne)
                else:
                    resultat = test(groupe, param.codage_des_lignes, i)
                    if resultat is None:
                        break
                    if resultat:
                        i += 1
                    groupe = [ligne]
            test(groupe, param.codage_des_lignes, i)
            
            
        angles = [{"angle": obj.val, "objet": obj} for obj in self.objets(False, type = Secteur_angulaire)]
        if angles:
            angles.sort() # attention, le classement d'un dictionnaire se fait selon l'ordre alphabtique des clefs
            groupe = [angles[0]]
            i = 2
            for angle in angles[1:]:
                if abs(groupe[-1]["angle"] - angle["angle"]) < param.tolerance:
                    groupe.append(angle)
                else:
#                    print abs(abs(groupe[-1]["angle"]) - math.pi/2)
                    if abs(abs(groupe[-1]["angle"]) - math.pi/2) < param.tolerance:
                        for elt in groupe:
                            elt["objet"].style(codage = "^")
                            elt["objet"].affiche()
                    else:
                        resultat = test(groupe, param.codage_des_angles, i)
                        if resultat is None:
                            break
                        if resultat:
                            i += 1
                    groupe = [angle]
            if abs(abs(groupe[-1]["angle"]) - math.pi/2) < param.tolerance:
                for elt in groupe:
                    elt["objet"].style(codage = "^")
                    elt["objet"].affiche()
            else:
                test(groupe, param.codage_des_angles, i)
                
        if self.canvas:
            self.canvas.actualiser()

    def relier_axe_x(self, point):
        if isinstance(point, Point):
            M = Point("%s.x" %point.nom, 0, fixe = True)
            M.label("{%s.x}" %point.nom, formule = True)
            s = Segment(point, M, style = ":")
            self.objets.add(M)
            self.objets.add(s)
    
    def relier_axe_y(self, point):
        if isinstance(point, Point):
            M = Point(0, "%s.y" %point.nom, fixe = True)
            M.label("{%s.y}" %point.nom, formule = True)
            s = Segment(point, M, style = ":")
            self.objets.add(M)
            self.objets.add(s)
    
    
    def relier_axes(self, point):
        self.relier_axe_x(point)
        self.relier_axe_y(point)
    #def ajouter_objets_temporaires(self, *objets):
    #    self.objets.__dict__["__objets_temporaires__"] += list(objets)
        
    #def effacer_objets_temporaires(self):
    #    self.objets.__dict__["__objets_temporaires__"] = []
        
    def objet_temporaire(self, objet = False):
        if objet is not False:
            if self.__objets_temporaires__:
                obj = self.__objets_temporaires__[0]
                self.canvas.graph.supprimer(obj.__representation__)
                self.canvas.actualiser()
            if objet is None:
                self.__objets_temporaires__ = []
            else:
                objet.__feuille__ = self
                self.__objets_temporaires__ = [objet]
        return self.__objets_temporaires__
        
        
    def contient_objet(self, objet):
        """contient_objet(self, objet) -> bool
        
        Teste rapidement si l'objet est rpertori dans la feuille.
        (Ne cherche pas parmi les objets temporaires.)"""
        
        return is_in(objet, self.objets.values())
        
        
    def contient_objet_temporaire(self, objet):
        """contient_objet_temporaire(self, objet) -> bool
        
        Teste rapidement si l'objet est rpertori comme objet temporaire dans la feuille."""
        
        for obj in self.__objets_temporaires__:
            if obj is objet:
                return True
        return False
        
        
    def point_temporaire(self):
        if self.__point_temporaire__ is None:
            self.__point_temporaire__ = Point()
            self.__point_temporaire__.__feuille__ = self
        return self.__point_temporaire__

    def start(self):
        u"Autorise le lancement d'animations."
        self.__stop__ = False
        
    def stop(self):
        u"Arrte toutes les animations en cours."
        self.__stop__ = True
        
    def animer(self, nom, debut = 0, fin = 1, pas = 0.02, periode = 0.03):
        u"Anime la variable nomme 'nom'."
        self.objets[nom].varier(debut, fin, pas, periode)

#######################################################################################
    
# Gestion de l'affichage

#    -> est largement gere desormais par le module MOD_affichage.py
    
    def message(self, messg): # A REECRIRE
        if self.canvas:
            messg = "Feuille %s - %s" %(self.nom, messg)
            self.canvas.message(messg)
        elif param.afficher_messages:
            print messg


        
#    def affiche(self):
#        for obj in self.objets():
#            try:
#                if not obj.nom().startswith("_"): obj.affiche()  # Les noms de la forme "_254" par exemple correspondent a des objets intermediaires utilises par la construction...
#            except:
#                self.message("Erreur lors de l'affichage de %s." % obj.nom())
#                if param.debug:
#                    print "Erreur lors de l'affichage de l'objet " + obj.nom()
#                    raise
        
        
        
    def refresh(self, *type):
        u"""A utiliser aprs une commande annuler ou refaire (affichage lent).
        
        Si 'type' est precis, ne rafraichit que les objets du ou des type(s) specifis.
        C'est utile en particulier pour rafraichir les droites.
        Quand on elargit la fentre d'affichage, il faut en effet
        redessiner les droites pour donner une impression d'infini !"""
        
        for objet in self.objets():
            if objet.etiquette: # il faut recalculer la position de toutes les tiquettes quand la fentre d'affichage change.
                objet.etiquette._recalculer = True
            if not type or isinstance(objet, type):
                try:
                    objet.affiche()
                except:
                    self.message(u"Erreur lors de l'affichage de %s." % objet.nom)
                    print_error(u"Erreur lors de l'affichage de l'objet " + objet.nom)
            else: # mme si tous les objets n'ont pas besoin d'tre rafraichis, leurs tiquettes doivent l'tre
                if objet.etiquette:
                    objet.etiquette.affiche()
        if self.parent:
            self.canvas.rafraichir_axes = True
            self.parent.affiche()
               
    
    
    def affiche(self):
        u"Affichage rapide."
        for objet in self.objets():
            self.canvas.graph.ajouter(objet.__representation__)
            self.canvas.graph.ajouter(objet.__trace__)
            if objet.etiquette:
                self.canvas.graph.ajouter(objet.etiquette.__representation__)
                
        
    def effacer_traces(self):
        u"Efface toutes les traces (sans enlever le mode trace des objets)."
        for objet in self.objets():
            self.canvas.graph.supprimer(objet.__trace__)
            objet.__trace__ = []
                
        self.canvas.actualiser()

    def objets_en_gras(self, *objets):
        u"""Met en gras les objets indiqus, et remet les autres objets en tat "normal" le cas chant."""
        
        changements = False
        for objet in self.objets():
            if objet in objets:
                val = objet.en_gras(True)
            else:
                val = objet.en_gras(False)
            if val is not None:
                changements = True
                
        if self.parent and changements:
            self.parent.canvas.actualiser()
#########################################################################################    


# Sauvegarde, exportation... du travail en cours.


        
##    def old_sauvegarder(self): #, creer_feuille = True):
##        u"Cre un fichier python qui permettra de recrer la figure et l'ensemble des objets."
##        
##        objets = [obj for obj in self.objets() if obj._enregistrer]
##        # on doit enregistrer les objets dans le bon ordre (suivant la _hierarchie) :
##        objets.sort(key=lambda x:getattr(x,"_hierarchie"))
##        texte = u"set_fenetre(" + str(self.fenetre())[1:-1] + ")\n"
##        texte += "\n".join([objet.nom() + "=" + repr(objet) for objet in objets]) + "\n"
##
##        for obj in objets: # Certaines precisions ne peuvent pas tre entres directement  la cration de l'objet
##            if obj.etiquette:
##                texte += u"%s.etiquette.style(**%s)\n" %(obj.nom(), obj.etiquette.style())
##            if isinstance(obj, Polygone): # les cotes sont crees directement par les Polygones, on met a jour leurs styles ici.
##                for i in xrange(len(obj.cotes)):
##                    texte += u"%s.cotes[%s].style(**%s)\n" %(obj.nom(), i, obj.cotes[i].style())
##                    texte += u"%s.cotes[%s].etiquette.style(**%s)\n" %(obj.nom(), i, obj.cotes[i].etiquette.style())
##                for i in xrange(len(obj.sommets)):
##                    texte += u"%s.sommets[%s].style(**%s)\n" %(obj.nom(), i, obj.sommets[i].style())
##                    texte += u"%s.sommets[%s].etiquette.style(**%s)\n" %(obj.nom(), i, obj.sommets[i].etiquette.style())
##        return texte    



##    def sauvegarder(self):
##        u"Renvoie l'ensemble des commandes python qui permettra de recrer la figure avec tous ses objets."
##        
##        texte = u"set_fenetre(" + str(self.fenetre())[1:-1] + ")\n"
##
##        objets = self.objets()
##        a_enregistrer = [obj for obj in objets if obj._enregistrer]
##        # on doit enregistrer les objets dans le bon ordre (suivant la _hierarchie) :
##        a_enregistrer.sort(key=lambda x:getattr(x,"_hierarchie"))
##        
##        # on fait un cas particulier pour les objets Texte, car ils peuvent contenir une formule
##        # qui dpend d'autres objets. Leur style n'est donc appliqu qu'aprs.
##        # (A CHANGER ?)
##        texte += u"\n".join(objet.nom + "=" + objet.__repr__(not isinstance(objet, Texte)) for objet in a_enregistrer) + "\n"
##        
##        texte += u"\n".join(obj.nom + u".style(**" + repr(obj.style()) + ")" for obj in objets if not obj._enregistrer or isinstance(obj, Texte)) + "\n"
##        
##        texte += u"\n".join(obj.nom + u".etiquette.style(**" + repr(obj.etiquette.style()) + ")" for obj in objets if obj.etiquette) + "\n"
##        
##        return texte    

 # version de dbut septembre 2008
##    def sauvegarder(self): 
##        u"Renvoie l'ensemble des commandes python qui permettra de recrer la figure avec tous ses objets."
##        
##        objets = self.objets()
##        # on doit enregistrer les objets dans le bon ordre (suivant la _hierarchie) :
##        objets.sort(key=lambda x:getattr(x,"_hierarchie"))
##        
##        texte = "fenetre = " + repr(self.fenetre) + "\n"
##        a_rajouter_a_la_fin = ""
##        deja_enregistres = set()
##        i = 0
##        
##        for objet in objets:
##            i += 1
##            if isinstance(objet, Texte) and objet.style("legende") == wxgeo.FORMULE:
##                texte += objet.nom + "=" + objet.__repr__(False) + "\n"
##                a_rajouter_a_la_fin += obj.nom + ".style(**" + repr(obj.style()) + ")\n"
##            elif isinstance(objet, Polygone):
##                for obj in objets[i:]:
##                    if isinstance(obj, (Cote, Sommet)) and obj.polygone is objet:
##                        texte += obj.nom + "=" + obj.__repr__(False) + "\n"
##                        deja_enregistres.add(obj)
##            elif isinstance(objet, Representant):
##               for obj in objets[i:]:
##                    if isinstance(obj, Extremite) and obj.representant is objet:
##                        texte += obj.nom + "=" + obj.__repr__(False) + "\n"
##                        deja_enregistres.add(obj)
##            elif isinstance(objet, (Sommet, Cote, Extremite)):
##                if not is_in(objet, deja_enregistres):
##                    texte += objet.nom + "=" + repr(objet) + "\n"
##            else:
##                texte += objet.nom + "=" + repr(objet) + "\n"
##        
##        for obj in objets:
##            if obj.etiquette is not None:
##                texte += obj.nom + ".etiquette.style(**" + repr(obj.etiquette.style()) + ")\n"
##            
##        return texte + a_rajouter_a_la_fin
##        
##        
##        # on fait un cas particulier pour les objets Texte, car ils peuvent contenir une formule
##        # qui dpend d'autres objets. Leur style n'est donc appliqu qu'aprs.
##        # (A CHANGER ?)
##        texte += u"\n".join(objet.nom + "=" + objet.__repr__(not isinstance(objet, Texte)) for objet in a_enregistrer) + "\n"
##        
##        texte += u"\n".join(obj.nom + u".style(**" + repr(obj.style()) + ")" for obj in objets if not obj._enregistrer or isinstance(obj, Texte)) + "\n"
##        
##        texte += u"\n".join(obj.nom + u".etiquette.style(**" + repr(obj.etiquette.style()) + ")" for obj in objets if obj.etiquette) + "\n"
##        
##        return texte    

# version de mi-septembre 2008
    def sauvegarder(self):
        u"Renvoie l'ensemble des commandes python qui permettra de recrer la figure avec tous ses objets."
        
        objets = self.objets()
        # on doit enregistrer les objets dans le bon ordre (suivant la _hierarchie) :
        objets.sort(key=lambda x:getattr(x,"_hierarchie"))
        
        texte = "fenetre = " + repr(self.fenetre) + "\n"
        a_rajouter_a_la_fin = ""
        
        for objet in objets:
            if isinstance(objet, Texte) and objet.style("legende") == wxgeo.FORMULE:
                # on fait un cas particulier pour les objets Texte, car ils peuvent contenir une formule
                # qui dpend d'autres objets. Leur style n'est alors appliqu qu'aprs.
                texte += objet.nom + "=" + objet.__repr__(False) + "\n"
                a_rajouter_a_la_fin += objet.nom + ".style(**" + repr(objet.style()) + ")\n"
            else:
                texte += objet.nom + "=" + repr(objet) + "\n"
        
        # Les tiquettes peuvent contenir des formules qui dpendent d'autres objets.
        for objet in objets:
            if objet.etiquette is not None:
                texte += objet.nom + ".etiquette.style(**" + repr(objet.etiquette.style()) + ")\n"
            
        return texte + a_rajouter_a_la_fin
        
        


    def effacer(self, refresh = True):
        self.objets.__effacer__()
        if refresh:
            self.refresh()
        
    def charger(self, commandes, refresh = True):
        u"""Excute un ensemble de commandes dans la feuille.
        
        Usage:
        f = Feuille()
        ...
        commandes = f.sauvegarder()
        f.effacer()
        f.charger(commandes)
        """    
        if self.canvas is not None:
            self.canvas.geler_affichage = True
        exec(commandes, self.objets)
        if self.canvas is not None:
            self.canvas.geler_affichage = False
#            self.canvas.actualiser()
        if refresh:
            self.refresh()
            
#########################################################################################    

# Gestion du zoom et des coordonnees, reglage de la fenetre d'affichage.

#    -> est gere desormais par le module MOD_affichage.py

    def reinitialiser_fenetre(self):
        self.__fenetre__ = param.fenetre
        self.refresh(Droite_generique, Interpolation_generique, Demidroite, Vecteur, Segment, Arc_generique, Widget)
    
    def _get_fenetre(self):            
        return self.__fenetre__
        
    def _set_fenetre(self, xmin_xmax_ymin_ymax):            
        xmin, xmax, ymin, ymax =xmin_xmax_ymin_ymax
            
        if xmin is None:
            xmin = self.__fenetre__[0]
        if xmax is None:
            xmax = self.__fenetre__[1]
        if ymin is None:
            ymin = self.__fenetre__[2]
        if ymax is None:
            ymax = self.__fenetre__[3]
        if abs(xmax - xmin) < 10*param.tolerance or abs(ymax - ymin) < 10*param.tolerance:
            self.erreur("Le reglage de la fenetre est incorrect.", ValueError)
        # Les 'float()' servent  contourner un bug de numpy 1.1.x et numpy 1.2.x (repr de float64)
        self.__fenetre__ = float(min(xmin, xmax)), float(max(xmin, xmax)), float(min(ymin, ymax)), float(max(ymin, ymax))
        self.refresh(Droite_generique, Interpolation_generique, Demidroite, Vecteur, Segment, Arc_generique, Widget)

    fenetre = property(_get_fenetre, _set_fenetre, reinitialiser_fenetre)

#########################################################################################    

# Diverses fonctionnalites de la feuille, utilisees par les objets.
# C'est un peu la boite a outils :-)




    def nom_aleatoire(self, prefixe = ALL.Objet._prefixe_nom): # utilise pour la generation d'objets de noms toujours differents
        #print self.objets.__noms__()
        liste = [int(s[len(prefixe):]) for s in self.objets.__noms__() if re.match(prefixe + "[0-9]+$", s)]
        #print liste
        return prefixe + str(max(liste or [0]) + 1)


    def erreur(self, message, erreur = None):
        self.message(u"Erreur : " + unicode2(message))
        if erreur is None:
            erreur = RuntimeError
        raise erreur, str2(message)


    def pause(self):
        wx.YieldIfNeeded()
        if self.__stop__:
            raise RuntimeError, "Interruption de la macro."
