# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Points                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from objet import *





################################################################################


## POINTS



class Point_generique(Objet_avec_coordonnees):
    u"""Un point gnrique.

    Usage interne : la classe mre pour tous les types de points (libres, barycentres, intersection...)."""

    _style_defaut = param.points
    _prefixe_nom = "M"
    
    def __init__(self, **styles):
        #~ self.__args = GestionnaireArguments()
        Objet.__init__(self, **styles)
        self.etiquette = ALL.Label_point(self)


    def _affiche(self):
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.plot()]
        x, y = self.coordonnees
        plot = self.__representation__[0]
        plot.set_data(pylab.array([x]), pylab.array([y]))
        plot._markeredgecolor = plot._markerfacecolor= self.style("couleur")
        plot._marker = self.style("style")
        plot._markersize = self.style("taille")
        plot._markeredgewidth = self.style("epaisseur")
        plot.zorder = self.style("niveau")


    def _espace_vital(self):
        x, y = self.coordonnees
        return (x, x, y, y)

    def _get_coordonnees(self):
        return NotImplemented    # sera implemente differemment pour chaque type de point

    @property
    def abscisse(self):
        return self.coordonnees[0]

    @property
    def ordonnee(self):
        return self.coordonnees[1]

    @property
    def affixe(self):
        coordonnees = self.coordonnees
        return coordonnees[0] + coordonnees[1]*1j

    x = abscisse
    y = ordonnee
    z = affixe

    def __sub__(self, y):
        return self + (-y)


    def __add__(self, y):
        if isinstance(y, ALL.Vecteur_generique): return Point_final(self, [y])
        raise TypeError, "vecteur attendu"

    # A>B est un alias de Vecteur(A,B) - attention aux parentheses pour 2*(A>B) !
    # A ne pas utiliser en interne (code peu lisible)
    def __gt__(self, point2):
        return ALL.Vecteur(self, point2)


    def _distance_inf(self, x, y, d):
        x0, y0 = self._pixel()
        return (x - x0)**2 + (y - y0)**2 < d**2


    def __eq__(self, y):
        if self.existe:
            if  isinstance(y, Point_generique) and y.existe:
                return abs(self.x - y.x) < param.tolerance and abs(self.y - y.y) < param.tolerance
            elif isinstance(y, (list, tuple, pylab.ndarray)) and len(y) == 2:
                return abs(self.x - y[0]) < param.tolerance and abs(self.y - y[1]) < param.tolerance
        return False

    def __ne__(self, y):
        return not (self == y)

    @staticmethod
    def _convertir(objet):
        if hasattr(objet, "__iter__"):
            return ALL.Point(*objet)
        raise TypeError, "'" + str(type(objet)) + "' object is not iterable"



class Point(Objet_avec_coordonnees_modifiables, Point_generique):
    u"""Un point libre.
    
    >>> A = Point(7,3); print A
    Point(x = 7.0, y = 3.0)
    """

    _style_defaut = param.points_deplacables

    abscisse = x = __x = Argument("Variable", defaut = 0)
    ordonnee = y = __y = Argument("Variable", defaut = 0)

    def __new__(cls, *args, **kw):
        if len(args) == 1:
            if isinstance(args[0], ALL.Droite_generique):
                newclass = ALL.Glisseur_droite
            elif isinstance(args[0], ALL.Segment):
                newclass = ALL.Glisseur_segment
            elif isinstance(args[0], ALL.Demidroite):
                newclass = ALL.Glisseur_demidroite
            elif isinstance(args[0], ALL.Cercle_generique):
                newclass = ALL.Glisseur_cercle
            elif isinstance(args[0], ALL.Arc_generique):
                newclass = ALL.Glisseur_arc_cercle
            else:
                return super(Point, cls).__new__(cls, *args, **kw)
        else:
            return super(Point, cls).__new__(cls, *args, **kw)
        glisseur = newclass.__new__(newclass, *args, **kw)
        glisseur.__init__(*args, **kw)
        return glisseur

    def __init__(self, x = None, y = None, **styles):
        x, y = self._recuperer_x_y(x, y)
        self.__x = x = Ref(x)
        self.__y = y = Ref(y)
        Objet_avec_coordonnees_modifiables.__init__(self, x, y, **styles)
        Point_generique.__init__(self, **styles)

 

    def _set_feuille(self):
            xmin, xmax, ymin, ymax = self.__feuille__.fenetre
            if "_Point__x" in self._valeurs_par_defaut:
                self.__x.val = module_random.uniform(xmin, xmax)
                self._valeurs_par_defaut.discard("_Point__x")
            if "_Point__y" in self._valeurs_par_defaut:
                self.__y.val = module_random.uniform(ymin, ymax)
                self._valeurs_par_defaut.discard("_Point__y")




    def _update(self, objet):
        if not isinstance(objet, Point):
            objet = self._convertir(objet)
        if isinstance(objet, Point):
            self.coordonnees = objet.coordonnees
        else:
            raise TypeError, "L'objet n'est pas un point."







##class Barycentre(Point_generique):
##    u"""Un barycentre.
##
##    Un barycentre de n points."""
##
##    points = __points = Arguments("Point_generique")
##    coeffs = __coeffs = Arguments("Variable")
##
##    def __init__(self, points, coeffs = None, **styles):
##        if coeffs is None:
##            coeffs = tuple(1 for point in points)
##        self.__points = points = tuple(Ref(obj) for obj in points)
##        self.__coeffs = coeffs = tuple(Ref(obj) for obj in coeffs)
##        # CTuple autorise une syntaxe style : G.points[0] = (1, 2)
##        # CTuple autorise une syntaxe style : G.coeffs[0] = 3.5
##        self.verifier_coeffs(coeffs)
##        Point_generique.__init__(self, **styles)
##
##
##    def _get_coordonnees(self):  # renvoie les coordonnes du barycentre en fonction de celles des points
##        coeffs = self.coefficients # valeur des coefficients
##        return tuple(builtin.sum(k*M.coo for k, M in zip(coeffs, self.__points))/self.__total)
##
##    def coefficients(self, coeffs = None):
##        if coeffs is not None:
##            self.verifier_coeffs(coeffs)
##            for coeff, valeur in zip(self.__coeffs, coeffs):
##                coeff.val = valeur # l'objet coeff (de type "Variable") reoit la valeur "valeur".
##        return self.__coeffs
##
##    coeffs = coefficients = property(coefficients, coefficients)
##
##    def verifier_coeffs(self, coeffs):
##        if len(self.__points) != len(coeffs):
##            self.erreur("nombre de coefficients incorrect.")
#### Ceci est inclus dans les conditions d'existence dsormais.            
###        if abs(sum(float(coeff) for coeff in coeffs)) < param.tolerance:
###            self.erreur("la somme des coefficients ne doit pas etre nulle.")
##
##    def _conditions_existence(self):
##        self.__total = sum(self.coefficients) # on le stocke pour eviter de le calculer 2 fois.
##        return [self.__total != 0]
##
##
##
##
##
##
##class Milieu(Barycentre):
##    u"""Un milieu.
##
##    Le milieu de 2 points"""
##
##    point1 = __point1 = Argument("Point_generique")
##    point2 = __point2 = Argument("Point_generique")
##
##    def __init__(self, point1, point2, **styles):
##        self.__point1 = point1 = Ref(point1)
##        self.__point2 = point2 = Ref(point2)
##        Barycentre.__init__(self, points = [point1, point2], **styles)


class Point_pondere(Objet):
    u"""Un point pondr.
    
    Usage interne : les points pondrs sont utiliss dans la dfinition des barycentres."""

    point = __point = Argument("Point_generique")
    coefficient = __coefficient = Argument("Variable")
    
    @staticmethod
    def _convertir(objet):
        if isinstance(objet, Point_generique):
            return Point_pondere(objet)
        elif hasattr(objet, "__iter__"):
            return Point_pondere(*objet)
        raise TypeError, "'" + str(type(objet)) + "' object is not iterable"

    def __init__(self, point, coefficient = None, **styles):
        if coefficient is None:
            coefficient = 1
        self.__point = point = Ref(point)
        self.__coefficient = coefficient = Ref(coefficient)
        Objet.__init__(self, **styles)
        
    def __iter__(self):
        return iter((self.__point, self.__coefficient))
        

class Barycentre(Point_generique):
    u"""Un barycentre.

    Un barycentre de n points."""

    points_ponderes = __points_ponderes = Arguments("Point_pondere")
    _prefixe_nom = "G"
    
    def __init__(self, *points_ponderes, **styles):                
        self.__points_ponderes = __points_ponderes = tuple(Ref(obj) for obj in points_ponderes)
        Point_generique.__init__(self, **styles)


    def _get_coordonnees(self):  # renvoie les coordonnes du barycentre en fonction de celles des points
        return tuple(sum(coeff*point.coo for point, coeff in self.__points_ponderes)/self.__total)

##    def coefficients(self, coeffs = None):
##        if coeffs is not None:
##            self.verifier_coeffs(coeffs)
##            for coeff, valeur in zip(self.__coeffs, coeffs):
##                coeff.val = valeur # l'objet coeff (de type "Variable") reoit la valeur "valeur".
##        return self.__coeffs
##
##    coeffs = coefficients = property(coefficients, coefficients)
##
##    def verifier_coeffs(self, coeffs):
##        if len(self.__points) != len(coeffs):
##            self.erreur("nombre de coefficients incorrect.")
#### Ceci est inclus dans les conditions d'existence dsormais.            
###        if abs(sum(float(coeff) for coeff in coeffs)) < param.tolerance:
###            self.erreur("la somme des coefficients ne doit pas etre nulle.")

    def _conditions_existence(self):
        self.__total = sum(point_pondere._Point_pondere__coefficient.val for point_pondere in self.__points_ponderes) # on le stocke pour eviter de le calculer 2 fois.
        return self.__total != 0






class Milieu(Barycentre):
    u"""Un milieu.

    Le milieu de 2 points"""

    _prefixe_nom = "I"
    
    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")

    def __init__(self, point1, point2, **styles):
        if isinstance(point1, ALL.Segment):
            point2 = point1._Segment__point2
            point1 = point1._Segment__point1
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        Barycentre.__init__(self, Point_pondere(point1, 1), Point_pondere(point2, 1), **styles)




class Point_final(Point_generique):
    u"""Un point dfini par une relation vectorielle.

    Point dfini par une relation vectorielle.
                              ->      ->     ->
    Exemple : point N tel que AN = -3 AB + 2 BC
    N = Point_final(A, [A>B, B>C], [-3,2])"""

    depart = __depart = Argument("Point_generique")
    vecteurs = __vecteurs = Arguments("Vecteur_generique")
    coeffs = coefficients = __coefficients = Arguments("Variable")

    def __init__(self, depart, vecteurs = (), coefficients = None, **styles):
        if coefficients == None:
            coefficients = tuple(1 for vecteur in vecteurs)
        self.__depart = depart = Ref(depart)
        self.__vecteurs = vecteurs = tuple(Ref(obj) for obj in vecteurs)
        self.__coefficients = coefficients = tuple(Ref(obj) for obj in coefficients)
        Point_generique.__init__(self, **styles)


##    def coefficients(self, coefficients = None):
##        if coefficients is not None:
##            for coeff, valeur in zip(self.__coefficients, coefficients):
##                coeff.val = valeur # l'objet coeff (de type "Variable") reoit la valeur "valeur".
##        return tuple(coeff.val for coeff in self.__coefficients)
##
##    coefficients = coefficients = property(coefficients, coefficients)

    def _get_coordonnees(self):
        return tuple(self.__depart.coo + sum(coeff*vecteur.coo for coeff, vecteur in zip(self.__coefficients, self.__vecteurs)))






class Point_translation(Point_generique):
    u"""Une image d'un point par translation."""

    point = __point = Argument("Point_generique")
    translation = __translation = Argument("Translation")

    def __init__(self, point, translation, **styles):
        self.__point = point = Ref(point)
        self.__translation = translation = Ref(translation)
        Point_generique.__init__(self, **styles)

    def _get_coordonnees(self):
        return tuple(self.__point.coo + self.__translation._Translation__vecteur.coo)




class Point_rotation(Point_generique):
    u"""Une image d'un point par rotation.

    Point construit  partir d'un autre via une rotation d'angle et de centre donn."""

    point = __point = Argument("Point_generique")
    rotation = __rotation = Argument("Rotation")

    def __init__(self, point, rotation, **styles):
        self.__point = point = Ref(point)
        self.__rotation = rotation = Ref(rotation)
        Point_generique.__init__(self, **styles)

    def _get_coordonnees(self):
        x0, y0 = self.__rotation.centre.coordonnees
        xA, yA = self.__point.coordonnees
        a = self.__rotation.radian
        sina = math.sin(a) ; cosa = math.cos(a)
        return (-sina*(yA - y0) + x0 + cosa*(xA - x0), y0 + cosa*(yA - y0) + sina*(xA - x0))

#    def _get_coordonnees2(self): # un poil plus lent
#        x0, y0 = self.__rotation.centre.coordonnees
#        xA, yA = self.__point.coordonnees
#        z = ((xA - x0) + (yA - y0)*1j)*cmath.exp(1j*self.__rotation.radian) + x0 + y0*1j
#        return z.real, z.imag





class Point_homothetie(Point_generique):
    u"""Une image d'un point par homothtie.

    Point construit  partir d'un autre via une homothtie de rapport et de centre donn."""

    point = __point = Argument("Point_generique")
    homothetie = __homothetie = Argument("Homothetie")

    def __init__(self, point, homothetie, **styles):
        self.__point = point = Ref(point)
        self.__homothetie = homothetie = Ref(homothetie)
        Point_generique.__init__(self, **styles)

    def _get_coordonnees(self):
        x0, y0 = self.__homothetie.centre.coordonnees
        xA, yA = self.__point.coordonnees
        k = self.__homothetie._Homothetie__rapport
        return x0 + k*(xA-x0), y0 + k*(yA-y0)






class Point_reflexion(Point_generique):
    u"""Une image d'un point par rflexion.

    Point construit  partir d'un autre via une symtrie d'axe donn."""

    point = __point = Argument("Point_generique")
    reflexion = __reflexion = Argument("Reflexion")

    def __init__(self, point, reflexion, **styles):
        self.__point = point = Ref(point)
        self.__reflexion = reflexion = Ref(reflexion)
        Point_generique.__init__(self, **styles)


    def _get_coordonnees(self):
        x0, y0 = self.__reflexion._Reflexion__droite._Droite_generique__point1
        x1, y1 = self.__reflexion._Reflexion__droite._Droite_generique__point2
        x, y = self.__point
        z = x1 - x0 + (y1 - y0)*1j
        M = (x - x0 + (y0 - y)*1j)*z/z.conjugate() + x0 + y0*1j
        return M.real, M.imag










class Projete_generique(Point_generique):
    u"""Un projet gnrique.

    Classe mre des diffrents types de projets orthogonaux (sur droite, sur cercle ou sur segment)."""

    def __init__(self, point, objet, **styles):
        #~ self._initialiser(point = Point_generique, objet = Objet)
        Point_generique.__init__(self, **styles)


    def distance(self):
        return self._distance(self, self.__point)







class Projete_droite(Projete_generique):
    u"""Un projet orthogonal sur une droite.

    Projet orthogonal d'un point sur une droite."""

    point = __point = Argument("Point_generique")
    droite = __droite = Argument("Droite_generique")

    def __init__(self, point, droite, **styles):
        self.__point = point = Ref(point)
        self.__droite = droite = Ref(droite)
        Projete_generique.__init__(self, point = point, objet = droite, **styles)

    def _get_coordonnees(self):
        xA, yA = self.__droite._Droite_generique__point1.coordonnees
        xB, yB = self.__droite._Droite_generique__point2.coordonnees
        x, y = self.__point.coordonnees
        xu = xB-xA #  u = vecteur A>B
        yu = yB-yA
        xu2 = xu*xu
        yu2 = yu*yu
        AB = xu2+yu2
        if AB > param.tolerance:
            return (yu2*xA+xu2*x+xu*yu*(y-yA))/AB, (xu2*yA+yu2*y+xu*yu*(x-xA))/AB
        else:
            return xA, yA






class Projete_cercle(Projete_generique):
    u"""Un projet orthogonal sur un cercle.

    Projet orthogonal d'un point sur un cercle."""

    point = __point = Argument("Point_generique")
    cercle = __cercle = Argument("Cercle_generique")

    def __init__(self, point, cercle, **styles):
        self.__point = point = Ref(point)
        self.__cercle = cercle = Ref(cercle)
        Projete_generique.__init__(self, point = point, objet = cercle, **styles)

    def _conditions_existence(self):
        # cas de non existence : si le point est au centre du cercle
        return self._carre_distance(self.__cercle.centre, self.__point) > param.tolerance**2


    def _get_coordonnees(self):
        k = angle_vectoriel((1, 0), self._vec(self.__cercle.centre, self.__point))
        return tuple(self.__cercle.centre.coo + self.__cercle.rayon*pylab.array((math.cos(k), math.sin(k))))








class Projete_arc_cercle(Projete_generique):
    u"""Un projet orthogonal sur un arc de cercle.

    Projet orthogonal d'un point sur un arc de cercle."""

    point = __point = Argument("Point_generique")
    arc = __arc = Argument("Arc_generique")

    def __init__(self, point, arc, **styles):
        self.__point = point = Ref(point)
        self.__arc = arc = Ref(arc)
        Projete_generique.__init__(self, point = point, objet = arc, **styles)

    def _conditions_existence(self):
        # cas de non existence : si le point est au centre du cercle
        return self._carre_distance(self.__arc.centre, self.__point) > param.tolerance**2


    def _get_coordonnees(self):
        M = self.__point
        arc = self.__arc
        O = arc.centre;
        a, b = arc._intervalle()
        c = angle_vectoriel((1, 0), self._vec(O, M))
        while c < a:
            c += 2*math.pi
        # La mesure d'angle c est donc dans l'intervalle [a; a+2*pi[
        if c > b: # c n'appartient pas  [a;b] (donc M est en dehors de l'arc de cercle)
            if c - b > 2*math.pi + a - c: # c est plus proche de a+2*pi
                c = a
            else:   # c est plus proche de b
                c = b
        if b - a < param.tolerance:
            return arc.point1.coordonnees

        k = (b - c)/(b - a)
        t = a*k + b*(1 - k)
        return tuple(arc.centre.coo + arc.rayon*pylab.array((math.cos(t), math.sin(t))))








class Projete_segment(Projete_generique):
    u"""Un projet orthogonal sur un segment.

    Projet orthogonal d'un point sur un segment."""

    point = __point = Argument("Point_generique")
    segment = __segment = Argument("Segment")

    def __init__(self, point, segment, **styles):
        self.__point = point = Ref(point)
        self.__segment = segment = Ref(segment)
        Projete_generique.__init__(self, point = point, objet = segment, **styles)



    def _get_coordonnees(self):
        xA, yA = A = self.__segment._Segment__point1
        xB, yB = B = self.__segment._Segment__point2
        x, y = self.__point.coordonnees
        AB2 = self._carre_distance(A, B)
        if AB2 > param.tolerance:
            k = produit_scalaire(self._vec((x,y), A), self._vec(B, A))/AB2
            k = min(max(k, 0), 1) # on se restreint au segment
            return (1-k)*xA+k*xB, (1-k)*yA+k*yB
        else: # A et B sont confondus
            return xA, yA







class Projete_demidroite(Projete_generique):
    u"""Un projet orthogonal sur une demi-droite.

    Projet orthogonal d'un point sur une demi-droite."""

    point = __point = Argument("Point_generique")
    demidroite = __demidroite = Argument("Demidroite")

    def __init__(self, point, demidroite, **styles):
        self.__point = point = Ref(point)
        self.__demidroite = demidroite = Ref(demidroite)
        Projete_generique.__init__(self, point = point, objet = demidroite, **styles)


    def _get_coordonnees(self):
        xA, yA = A = self.__demidroite._Ligne_generique__point1
        xB, yB = B = self.__demidroite._Ligne_generique__point2
        x, y = self.__point.coordonnees
        AB2 = self._carre_distance(A, B)
        if AB2 > param.tolerance:
            k = produit_scalaire(self._vec((x,y), A), self._vec(B, A))/AB2
            k = max(k, 0) # on se restreint  la demi-droite
            return (1-k)*xA+k*xB, (1-k)*yA+k*yB
        else: # A et B sont confondus
            return xA, yA






class Centre_polygone_generique(Point_generique):
    u"""Un centre d'un triangle.

    Classe mre des diffrents centres d'un triangle."""


    def __init__(self, polygone, **styles):
        #~ self._initialiser(polygone = Polygone)
        Point_generique.__init__(self, **styles)






class Centre_gravite(Centre_polygone_generique):
    u"""Un centre de gravit.

    Centre de gravit d'un polygone (l'intersection des mdianes dans le cas d'un triangle)."""

    _prefixe_nom = "G"

    polygone = __polygone = Argument("Polygone_generique")

    def __init__(self, polygone, **styles):
        self.__polygone = polygone = Ref(polygone)
        Centre_polygone_generique.__init__(self, polygone, **styles)

    def _conditions_existence(self):
        return self.__polygone.centre_gravite.existe

    def _get_coordonnees(self):
        return self.__polygone.centre_gravite.coordonnees






class Orthocentre(Centre_polygone_generique):
    u"""Un orthocentre.

    Orthocentre d'un triangle (intersection des hauteurs)."""

    _prefixe_nom = "H"

    triangle = __triangle = Argument("Triangle")

    def __init__(self, triangle, **styles):
        self.__triangle = triangle = Ref(triangle)
        Centre_polygone_generique.__init__(self, triangle, **styles)

    def _conditions_existence(self):
        return self.__triangle.orthocentre.existe

    def _get_coordonnees(self):
        return self.__triangle.orthocentre.coordonnees





class Centre_cercle_circonscrit(Centre_polygone_generique):
    u"""Un centre du cercle circonscrit.

    Centre du cercle circonscrit d'un triangle (intersection des mdiatrices)."""

    _prefixe_nom = "O"
    
    triangle = __triangle = Argument("Triangle")

    def __init__(self, triangle, **styles):
        self.__triangle = triangle = Ref(triangle)
        Centre_polygone_generique.__init__(self, triangle, **styles)

    def _conditions_existence(self):
        return self.__triangle.centre_cercle_circonscrit.existe

    def _get_coordonnees(self):
        return self.__triangle.centre_cercle_circonscrit.coordonnees





class Centre_cercle_inscrit(Centre_polygone_generique):
    u"""Un centre du cercle inscrit.

    Centre du cercle inscrit d'un triangle (intersection des bissectrices)."""

    _prefixe_nom = "I"

    triangle = __triangle = Argument("Triangle")

    def __init__(self, triangle, **styles):
        self.__triangle = triangle = Ref(triangle)
        Centre_polygone_generique.__init__(self, triangle, **styles)

    def _conditions_existence(self):
        return self.__triangle.centre_cercle_inscrit.existe

    def _get_coordonnees(self):
        return self.__triangle.centre_cercle_inscrit.coordonnees





class Centre(Point_generique):
    u"""Un centre de cercle."""

    _prefixe_nom = "O"

    cercle = __cercle = Argument("Cercle_generique")

    def __init__(self, cercle, **styles):
        self.__cercle = cercle = Ref(cercle)
        Point_generique.__init__(self, **styles)

#    def _conditions_existence(self):
#        return [self.__cercle.existe]

    def _get_coordonnees(self):
        a, b, c = self.__cercle.equation
        return (-a/2, -b/2)





class Point_equidistant(Point_generique):
    u"""Point quidistant de 3 points.

    Utilis surtout pour un usage interne, le calcul des coordonnes est plus rapide qu'en passant par les mdiatrices."""

    point1 = __point1 = Argument("Point_generique")
    point2 = __point2 = Argument("Point_generique")
    point3 = __point3 = Argument("Point_generique")

    def __init__(self, point1, point2, point3, **styles):
        self.__point1 = point1 = Ref(point1)
        self.__point2 = point2 = Ref(point2)
        self.__point3 = point3 = Ref(point3)
        Point_generique.__init__(self, **styles)

    def _conditions_existence(self):
        xA, yA = self.__point1.coordonnees
        xB, yB = self.__point2.coordonnees
        xC, yC = self.__point3.coordonnees
        a = xB - xA
        b = yB - yA
        c = xC - xB
        d = yC - yB
        self.__det2 = 2*(a*d - b*c)
        return abs(self.__det2) > param.tolerance or abs(xB - xC) + abs(yB - yC) < param.tolerance or abs(xA - xC) + abs(yA - yC) < param.tolerance or abs(xB - xA) + abs(yB - yA) < param.tolerance

    def _get_coordonnees(self):
        xA, yA = self.__point1.coordonnees
        xB, yB = self.__point2.coordonnees
        xC, yC = self.__point3.coordonnees
        if abs(self.__det2) > param.tolerance:
            a = xB - xA
            b = yB - yA
            c = xC - xB
            d = yC - yB
            p1 = a*d; p2 = b*c
            return (p1*(xA+xB) - p2*(xB+xC) - b*d*(yC-yA))/self.__det2, (a*c*(xC-xA) - p2*(yA+yB) + p1*(yB+yC))/self.__det2
        elif abs(xB - xC) + abs(yB - yC) < param.tolerance:
            return (.5*(xA+xB), .5*(yA+yB))
        elif abs(xA - xC) + abs(yA - yC) < param.tolerance:
            return (.5*(xA+xB), .5*(yA+yB))
        elif abs(xB - xA) + abs(yB - yA) < param.tolerance:
            return (.5*(xA+xC), .5*(yA+yC))







##########################################################################################

# Glisseurs





class Glisseur_generique(Point_generique):
    u"""Un glisseur gnrique.

    Classe mre des diffrents types de glisseurs"""
    
    _style_defaut = param.points_deplacables
    
    objet = __objet = Argument("Objet")
    k = __k = Argument("Variable")

    def __init__(self, objet, k, **styles):
        self.__objet = objet = Ref(objet)
        self.__k = k = Ref(k)
        Point_generique.__init__(self, **styles)

    def _initialiser_k(self, k):
        if isinstance(k, (list, tuple)):
            k = self._conversion_coordonnees_parametre(*k)
        return k

    def _get_coordonnees(self):
        return tuple(self.coo)


    def _set_coordonnees(self, x = None, y = None):
        if x is not None:
#            self._recalculer = True # inutile ?
            self.k = self._conversion_coordonnees_parametre(x, y)


class Glisseur_vecteur(Glisseur_generique):
    u"""Un glisseur sur vecteur."""

    vecteur = __vecteur = Argument("Vecteur")
    parametre = k = __k = Argument("Variable", None, lambda obj, value: max(min(value, 1), 0))

    def __init__(self, vecteur, k = None, **styles):
        if k is None:
            k = module_random.uniform(0, 1)
        self.__vecteur = vecteur = Ref(vecteur)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_generique.__init__(self, vecteur, k, **styles)

    @property
    def coo(self):
        return (1 - self.__k.val)*self.__vecteur._Vecteur__point1.coo + self.__k.val*self.__vecteur._Vecteur__point2.coo

    def _conversion_coordonnees_parametre(self, x, y):
        A = self.__vecteur._Vecteur__point1; B = self.__vecteur._Vecteur__point2
        return produit_scalaire(self._vec((x,y), A), self._vec(B, A))/self._carre_distance(A, B)



class Glisseur_ligne_generique(Glisseur_generique):
    u"""Un glisseur gnrique sur ligne.

    Classe mre des diffrents types de glisseurs sur ligne (ie. droite, segment, demi-droite)"""

    ligne = __ligne = Argument("Ligne_generique")
    k = __k = Argument("Variable")

    def __init__(self, ligne, k, **styles):
        self.__ligne = ligne = Ref(ligne)
        self.__k = k = Ref(k)
        Glisseur_generique.__init__(self, ligne, k, **styles)

    @property
    def coo(self):
        return (1 - self.__k.val)*self.__ligne._Ligne_generique__point1.coo + self.__k.val*self.__ligne._Ligne_generique__point2.coo





class Glisseur_droite(Glisseur_ligne_generique):
    u"""Un point sur une droite.

    Point pouvant 'glisser' sur une droite. k est un coefficient barycentrique.
    Si A et B sont les deux points de rfrence de la droite, k le coefficient,
    les coordonnes du glisseur M seront determines par la formule M = kA + (1-k)B."""

    droite = __droite = Argument("Droite_generique")
    parametre = k = __k = Argument("Variable")

    def __init__(self, droite, k = None, **styles):
        if k is None:
            k = module_random.normalvariate(0.5, 0.5)
        self.__droite = droite = Ref(droite)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_ligne_generique.__init__(self, ligne = droite, k = k, **styles)



    def _conversion_coordonnees_parametre(self, x, y):
        A = self.__droite._Ligne_generique__point1; B = self.__droite._Ligne_generique__point2
        return produit_scalaire(self._vec((x,y), A), self._vec(B, A))/self._carre_distance(A, B)







class Glisseur_segment(Glisseur_ligne_generique):
    u"""Un point sur un segment.

    Point pouvant 'glisser' sur un segment. k est un coefficient barycentrique.
    Si A et B sont les deux points de rfrence de la droite, k le coefficient,
    les coordonnes du glisseur M seront determines par la formule M = kA + (1-k)B."""

    segment = __segment = Argument("Segment")
    parametre = k = __k = Argument("Variable", None, lambda obj, value: max(min(value, 1), 0))
    # la valeur du parametre doit rester comprise entre 0 et 1 (segment)

    def __init__(self, segment, k = None, **styles):
        if k is None:
            k = module_random.uniform(0, 1)
        self.__segment = segment = Ref(segment)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_ligne_generique.__init__(self, ligne = segment, k = k, **styles)


    def _conversion_coordonnees_parametre(self, x, y):
        A = self.__segment._Ligne_generique__point1; B = self.__segment._Ligne_generique__point2
        return produit_scalaire(self._vec((x,y), A), self._vec(B, A))/self._carre_distance(A, B)





class Glisseur_demidroite(Glisseur_ligne_generique):
    u"""Un point sur une demi-droite.

    Point pouvant 'glisser' sur un segment. k est un coefficient barycentrique.
    Si A et B sont les deux points de rfrence de la droite, k le coefficient,
    les coordonnes du glisseur M seront determines par la formule M = kA + (1-k)B."""

    demidroite = __demidroite = Argument("Demidroite")
    parametre = k = __k = Argument("Variable", None, lambda obj, value: max(value, 0))

    def __init__(self, demidroite, k = None, **styles):
        if k is None:
            k = abs(normalvariate(0.5, 0.5))
        self.__demidroite = demidroite = Ref(demidroite)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_ligne_generique.__init__(self, ligne = demidroite, k = k, **styles)

##    def parametre(self, coeff = None):
##        u"""Alias plus explicite de self.k."""
##        if coeff is not None:
##            self.__k.val = max(coeff, 0) # la valeur du parametre doit rester positive (demi-droite)
##        return self.__k.val
##
##    k = parametre = property(parametre, parametre)

    def _conversion_coordonnees_parametre(self, x, y):
        A = self.__demidroite._Ligne_generique__point1; B = self.__demidroite._Ligne_generique__point2
        return produit_scalaire(self._vec((x,y), A), self._vec(B, A))/self._carre_distance(A, B)




class Glisseur_cercle(Glisseur_generique):
    u"""Un point sur un cercle.

    Point pouvant 'glisser' sur un cercle. k est un angle.
    Si O et r sont les centres et rayons du cercle, k l'angle,
    les coordonnes du glisseur M seront dtermines par la formule M = O + r*(cos(k), sin(k))."""

    cercle = __cercle = Argument("Cercle_generique")
    parametre = k = __k = Argument("Variable")

    def __init__(self, cercle, k = None, **styles):
        if k is None:
            k = module_random.uniform(0, 2*math.pi)
        self.__cercle = cercle = Ref(cercle)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_generique.__init__(self, objet = cercle, k = k, **styles)


    @property
    def coo(self):
        cercle = self.__cercle
        k = self.__k
        return cercle.centre.coo + cercle.rayon*pylab.array((math.cos(k), math.sin(k)))


    def _conversion_coordonnees_parametre(self, x, y):
        O = self.__cercle.centre
        M = (x, y)
        if O != M:
            return angle_vectoriel((1, 0), self._vec(O, M))
        else:
            return  0








class Glisseur_arc_cercle(Glisseur_generique):
    u"""Un point sur un arc de cercle.

    Point pouvant 'glisser' sur un cercle. k est un angle.
    Si O et r sont les centres et rayons de l'arc, k un coefficient compris entre 0 et 1,
    et a et b les mesures d'angle marquant le dbut et la fin de l'arc,
    les coordonnes du glisseur M seront dtermines par la formule M = O + r*(cos(ka+(1-k)b), sin(ka+(1-k)b)),
    si l'arc est de sens direct, et M = O + r*(cos(kb+(1-k)a), sin(kb+(1-k)a)) sinon."""

    arc = __arc = Argument("Arc_generique")
    parametre = k = __k = Argument("Variable")

    def __init__(self, arc, k = None, **styles):
        if k is None:
            k = module_random.uniform(0, 1)
        self.__arc = arc = Ref(arc)
        self.__k = k = Ref(self._initialiser_k(k))
        Glisseur_generique.__init__(self, objet = arc, k = k, **styles)


    @property
    def coo(self):
        arc = self.__arc
        a, b = arc._intervalle()
        k = self.__k
        if arc._sens() == 1:
            t = a*k + b*(1 - k)
        else:
            t = b*k + a*(1 - k)
        return arc.centre.coo + arc.rayon*pylab.array((math.cos(t), math.sin(t)))


    def _conversion_coordonnees_parametre(self, x, y):
        M = (x, y)
        O = self.__arc.centre;
        if O != M:
            a, b = self.__arc._intervalle()
            c = angle_vectoriel((1, 0), self._vec(O, M))
            while c < a:
                c += 2*math.pi
            # La mesure d'angle c est donc dans l'intervalle [a; a+2*pi[
            if c > b: # c n'appartient pas  [a;b] (donc M est en dehors de l'arc de cercle)
                if c - b > 2*math.pi + a - c: # c est plus proche de a+2*pi
                    c = a
                else:   # c est plus proche de b
                    c = b
            if b - a:
                if self.__arc._sens() == 1:
                    return (b - c)/(b - a)
                else:
                    return (c - a)/(b - a)
            else:
                return 0.5
        else:
            return 0



