# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

from customtest import *





class TestGeolibLignes(CustomTest):

    def test_Segment(self):
        A = ALL.Point(4.5,  7.3)
        B = ALL.Point(4,  2.1)
        s = ALL.Segment(A,  B)
        self.assert_(isinstance(s.etiquette, ALL.Label_segment))
        self.assertAlmostEqual(s.longueur, math.sqrt((B.x - A.x)**2 + (B.y - A.y)**2))
        I = ALL.Milieu(s.point1,  s.point2)
        self.assertEqual(I.coordonnees,  ((A.x+B.x)/2, (A.y+B.y)/2))
        M = ALL.Barycentre((A,  1),  (B,  -2))
        N = ALL.Barycentre((A,  -2),  (B,  1))
        self.assert_(I in s)
        self.assert_(M not in s)
        self.assert_(N not in s)
        self.assert_(s.style("legende") == ALL.param.RIEN)

    def test_Demidroite(self):
        A = ALL.Point(4.5,  7.3)
        B = ALL.Point(4,  2.1)
        s = ALL.Demidroite(A,  B)
        self.assert_(isinstance(s.etiquette, ALL.Label_demidroite))
        self.assertRaises(AttributeError,  getattr,  s, "longueur")
        I = ALL.Milieu(s.origine,  s.point)
        self.assertEqual(I.coordonnees,  ((A.x+B.x)/2, (A.y+B.y)/2))
        M = ALL.Barycentre((A,  1),  (B,  -2))
        N = ALL.Barycentre((A,  -2),  (B,  1))
        self.assert_(I in s)
        self.assert_(M in s)
        self.assert_(N not in s)
        self.assert_(s.style("legende") == ALL.param.RIEN)
        
    def test_Droite(self):
        A = ALL.Point(4.5,  7.3)
        B = ALL.Point(4,  2.1)
        d = ALL.Droite(A,  B)
        self.assert_(isinstance(d.etiquette, ALL.Label_droite))
        self.assertRaises(AttributeError,  getattr,  d, "longueur")
        I = ALL.Milieu(d.point1,  d.point2)
        self.assertEqual(I.coordonnees,  ((A.x+B.x)/2, (A.y+B.y)/2))
        M = ALL.Barycentre((A,  1),  (B,  -2))
        N = ALL.Barycentre((A,  -2),  (B,  1))
        self.assert_(I in d)
        self.assert_(M in d)
        self.assert_(N in d)
        self.assert_(isinstance(d.equation,  tuple))
        self.assert_(d.style("legende") == ALL.param.RIEN)
        # Test du typage dynamique
        d = ALL.Droite("y=x+1")
        self.assert_(ALL.Point(0, 1) in d)
        d = ALL.Droite(ALL.Point(1, 2), ALL.Vecteur_libre(1, 1))
        self.assert_(ALL.Point(1, 2) in d)
        self.assert_(ALL.Point(2, 3) in d)


    def test_Droite_vectorielle(self):
        v = ALL.Vecteur_libre(1,  7)
        A = ALL.Point(-2, 3)
        d = ALL.Droite_vectorielle(A,  v)
        self.assert_(d.vecteur is v and d.point is A)
        self.assertAlmostEqual(v.y/v.x,  -d.equation[0]/d.equation[1])
        B = rand_pt()
        d1 = ALL.Droite_vectorielle(B, v)
        self.assert_(d.parallele(d1))

    def test_Parallele(self):
        d0 = ALL.Droite_equation(2,  1,  7)
        A = ALL.Point(-2, 3)
        d = ALL.Parallele(d0,  A)
        self.assert_(d.parallele(d0))
        self.assert_(d.droite is d0 and d.point is A)
        self.assertAlmostEqual(d0.equation[:1],  d.equation[:1])

##    def test_Droite_rotation(self):
##        r = ALL.Rotation(ALL.Point(1.45, -2.59), math.pi/3)
##        C = ALL.Point(1.458, -5.255)
##        D = ALL.Point(3.478, -2.14788)
##        d = ALL.Droite(C, D)
##        # Dans ce qui suit, d1, d2 et d3 doivent correspondre  la mme droite.
##        d1 = ALL.Droite_rotation(d,  r)
##        d2 = ALL.Droite(r(C), r(D))
##        d3 = r(d)
##        a, b, c = d1.equation
##        self.assertAlmostEqual(d1.equation_reduite, d2.equation_reduite)
##        self.assertAlmostEqual(d1.equation_reduite, d3.equation_reduite)
##        self.assertAlmostEqual(d1.equation_reduite, (-a/b, -c/b))
##        d = ALL.Droite_rotation(ALL.Droite_equation(1, -1, 1),  ALL.Rotation(ALL.Point(0, 0), math.pi/2))
##        a, b, c = d.equation
##        self.assertAlmostEqual(b/a,  1)
##        self.assertAlmostEqual(c/a,  1)
        

    def test_Mediatrice(self):
        A = ALL.Point(4.5,  7.3)
        B = ALL.Point(-4.147,  2.1)
        s = ALL.Segment(A,  B)
        d0 = ALL.Mediatrice(s)
        d1 = ALL.Mediatrice(A,  B)
        I = ALL.Milieu(A,  B)
        self.assert_(I in d0)
        self.assert_(I in d1)
        a,  b,  c = s.equation
        a0,  b0,  c0 = d0.equation
        self.assertAlmostEqual(a*a0 + b*b0,  0)
        self.assertAlmostEqual(d0.equation,  d1.equation)

    def test_Perpendiculaire(self):
        d = ALL.Droite_equation(-1, 2, 0)
        M = ALL.Point()
        d0 = ALL.Perpendiculaire(d,  M)
        a,  b,  c = d.equation
        a0,  b0,  c0 = d0.equation
        self.assert_(d.perpendiculaire(d0))
        self.assertAlmostEqual(a*a0 + b*b0,  0)
        self.assert_(M in d0)

    def test_Droite_equation(self):
        a = randint(50) - randint(50) + 0.1 # afin que a ne soit pas nul
        b = randint(50) - randint(50) + random()
        c = randint(50) - randint(50) + random()
        d, e, f = ALL.Droite_equation(a, b, c).equation
        self.assertAlmostEqual((e/d, f/d),  (b/a, c/a))
        self.assertEqual(ALL.Droite_equation(a, 0, 0).equation[1:],  (0, 0))
        self.assertEqual((ALL.Droite_equation(0, a, 0).equation[0], ALL.Droite_equation(0, a, 0).equation[2]),  (0, 0))
        self.assert_(not ALL.Droite_equation(0, 0, 0).existe)
        d = ALL.Droite_equation("y=-5/2x-3/2")
        self.assert_(ALL.Point(0, -1.5) in d)
        self.assert_(ALL.Point(-1, 1) in d)
        d = ALL.Droite_equation("x=2*10**2")
        self.assert_(ALL.Point(200, -1000) in d)
        self.assert_(ALL.Point(100, -1000) not in d)
        d = ALL.Droite_equation("2*x+2*y=1")
        self.assert_(ALL.Point(0.5, 0) in d)
        self.assert_(ALL.Point(1, -0.5) in d)

    def test_Bissectrice(self):
        A = ALL.Point(1, -5)
        B = ALL.Point(1.5, -5.3)
        C = ALL.Point(3, -4)
        d = ALL.Bissectrice(A,  B,  C)
        a, b, c = d.equation
        d,  e = (0.0870545184921, -1.03861105199)
        self.assertAlmostEqual(b/a, d)
        self.assertAlmostEqual(c/a, e)

    def test_Tangente(self):
        A = ALL.Point(4.75, -2.56887)
        O = ALL.Point(2.56874, -85.2541)
        M = ALL.Point(7.854, -552.444)
        c = ALL.Cercle_rayon(O, A)
        d = ALL.Tangente(c, A)
        self.assert_(A in d)
        self.assert_(M not in d)
        d1 = ALL.Tangente(c, M)
        self.assert_(M in d1)
        self.assert_(A not in d1)
        self.assert_(not  ALL.Tangente(c, O).existe)
        


        
        
if __name__ == '__main__':
    unittest.main()
