# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                    Texte                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

from objet import *








class Texte(Objet_avec_coordonnees_modifiables):
    u"""Un texte.
    
    Un texte  afficher"""
    
    _style_defaut = param.textes
    _prefixe_nom = "t"
    
##    try:
##        __empty_font__ = matplotlib.font_manager.FontProperties()
##    except NameError:
##        pass
    
##    __weight_dict = {400: 'light', 500: 'normal', 600: 'semibold', 700: 'bold', 800: 'heavy', 900: 'black'}
    
    def _set_texte(self, value):
        if isinstance(value, basestring) and value != "" and hasattr(self, "_style"):
            value = self.tester_value(value)
            self.style(label = value)
        return value
    
    def _get_texte(self, value):
        return self.style("label")
    
    texte = __texte = Argument("basestring", _get_texte, _set_texte)
    abscisse = x = __x = Argument("Variable", defaut = 0)
    ordonnee = y = __y = Argument("Variable", defaut = 0)

    def __init__(self, texte = "", x = None, y = None, **styles):
        x, y = self._recuperer_x_y(x, y)
        if param.unicode:
            texte = unicode2(texte, param.encodage)

        if texte != "":
            texte = self.tester_value(texte)
            styles["label"] = texte
        else:
            if styles.has_key("label"):
                styles["label"] = self.tester_value(styles["label"])

        self.__texte = texte = Ref(texte)
        self.__x = x = Ref(x)
        self.__y = y = Ref(y)

        Objet_avec_coordonnees_modifiables.__init__(self, x, y, **styles)
        
    def style(self, *args, **kw):
        if kw.has_key("label"):
            kw["label"] = self.tester_value(kw["label"])
        return Objet_avec_coordonnees_modifiables.style(self, *args, **kw)
     
    def tester_value(self, value):
        if "$" in value:
            try:
                mathtext_parser(value)
            except Exception:
                # Pour viter de planter tout l'affichage de matplotlib
                print_error()
                return value.replace("$", "#")
        return value


    def _affiche(self):
        #self.__canvas__.Texte(self.texte, self.coo())
        if not self.__representation__:
            self.__representation__ = [self.__canvas__.text(), self.__canvas__.fill()]
            #~ self.__representation__[0]._fontproperties = module_copy.copy(self.__empty_font__)
            self.__representation__[0]._fontproperties = matplotlib.font_manager.FontProperties()
        x, y = self.coordonnees
        text = self.__representation__[0]
        text.set_x(x)
        text.set_y(y)
        text._color = self.style("couleur")
        font = text._fontproperties
        font.set_size(self.style("taille")*param.zoom_texte)
        #font.set_stretch(self.style("largeur"))  # mal gere par matploltib (version 0.87)
        
        if param.latex:  # formatage gr par LaTeX
            texte = self.label()
            style = self.style("style")
            if  style == "italic":
                texte = "\\textit{" + texte + "}"
            elif style == "oblique":
                texte = "\\textsl{" + texte + "}"
            if self.style("epaisseur") == "gras":
                texte = "\\textbf{" + texte + "}"
            famille = self.style("famille")
            if famille == "serif":
                texte = "\\rmfamily{" + texte + "}"
            elif famille == "monospace":
                texte = "\\ttfamily{" + texte + "}"
#            elif famille == "cursive":
#                texte = "\\mathscr{" + texte + "}"
            elif famille != "sans-serif":
                warning("Famille de police non disponible en mode LaTeX.")
            text._text = texte
        else: # formatage gr par matplotlib
#            font.set_weight(self.style("epaisseur") == "gras" and "bold" or "normal")
            font.set_weight(self.style("epaisseur") > 55 and "bold" or "normal")
            font.set_style(self.style("style"))
            font.set_family(self.style("famille"))
            text._text = self.label()
        
        text._rotation = self.style("angle")
        text._verticalalignment = self.style("alignement_vertical")
        text._horizontalalignment = self.style("alignement_horizontal")
        text.zorder = self.style("niveau") + .001
        fill = self.__representation__[1]
        if self.style("fond") is None:
            fill._visible = False
        else:
            fill._visible = True
            can = self.__canvas__
            box = text.get_window_extent(can.get_renderer())
            if param.old_matplotlib:
                w = .5*box.width()*can.coeff(0)
                h = .5*box.height()*can.coeff(1)
            else:
                w = .5*box.width*can.coeff(0)
                h = .5*box.height*can.coeff(1)                
            if text._verticalalignment == "left":
                x += w 
            elif text._verticalalignment == "right":
                x -= w
            if text._horizontalalignment == "top":
                y -= h
            elif text._horizontalalignment == "bottom":
                y += h
            fill.xy = [(x - w, y - h), (x - w, y + h), (x + w, y + h), (x + w, y - h)]
            fill._facecolor = fill._edgecolor = self.style("fond")
            fill.zorder = self.style("niveau")
        #debug(font.__dict__)

    def _boite(self):
        # Note : ymin et ymax "permutent" souvent car les transformations appliques inversent l'orientation.
        can = self.__canvas__
        l, h = can.dimensions
        box = self.__representation__[0].get_window_extent(can.get_renderer())
        if param.old_matplotlib:
            xmin = box.xmin()
            ymax = h - box.ymin()
            xmax = box.xmax()
            ymin = h - box.ymax()
        else:
            xmin = box.xmin
            ymax = h - box.ymin
            xmax = box.xmax
            ymin = h - box.ymax            
        return xmin, xmax, ymin, ymax

    def _espace_vital(self):
        # Note : ymin et ymax "permutent" souvent car les transformations appliques inversent l'orientation.
##        can = self.__canvas__
##        l, h = can.dimensions
##        box = self.__representation__[0].get_window_extent(can.get_renderer())
##        xmin = box.xmin()
##        ymax = h - box.ymin()
##        xmax = box.xmax()
##        ymin = h - box.ymax()
        can = self.__canvas__
        _xmin, _xmax, _ymin, _ymax = self._boite()
        xmin, ymin = can.pix2coo(_xmin, _ymax)
        xmax, ymax = can.pix2coo(_xmax, _ymin)
        return xmin, xmax, ymin, ymax
     


    def _distance_inf(self, x, y, d):
##        can = self.__canvas__
##        l, h = can.dimensions
##        box = self.__representation__[0].get_window_extent(can.get_renderer())
##        xmin = box.xmin()
##        ymax = h - box.ymin()
##        xmax = box.xmax()
##        ymin = h - box.ymax() 
        xmin, xmax, ymin, ymax = self._boite()
        return xmin - d < x < xmax + d and ymin - d < y < ymax + d
        

    @staticmethod
    def _convertir(objet):
        if isinstance(objet, basestring):
            return Texte(objet)
        if hasattr(objet, "__iter__"):
            return Texte(*objet)
        raise TypeError, "object is not iterable."
        
        
    def _update(self, objet):
        if not isinstance(objet, Texte):
            if hasattr(objet, "__iter__") and len(objet) == 1 and isinstance(objet[0], basestring):
                self.label(objet[0])
                return
            else:
                objet = self._convertir(objet)
        if isinstance(objet, Texte):
            self.label(objet.texte)
            self.coordonnees = objet.coordonnees
        else:
            raise TypeError, "l'objet n'est pas un texte."


    def _set_feuille(self):
        xmin, xmax, ymin, ymax = self.__feuille__.fenetre
        if "_Texte__x" in self._valeurs_par_defaut:
            self.__x.val = module_random.uniform(xmin, xmax)
            self._valeurs_par_defaut.discard("_Point__x")
        if "_Texte__y" in self._valeurs_par_defaut:
            self.__y.val = module_random.uniform(ymin, ymax)
            self._valeurs_par_defaut.discard("_Point__x")

    def _en_gras(self, valeur):
        if self.__representation__:
            if valeur:
                self.__representation__[0]._bbox = {'alpha': 0.5, 'linewidth': 1, 'fill': False}
            else:
                self.__representation__[0]._bbox = None
                
