# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#                   Objets                    #
##--------------------------------------#######
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode


from points import *







##########################################################################################

# Divers




class Transformation_generique(Objet):
    u"""Une transformation.

    La classe mre de toutes les transformations (usage interne)."""
    def __init__(self, **styles):
        Objet.__init__(self, **styles)

    def __call__(self, objet):
        if isinstance(objet, Ref):
            if isinstance(objet.__objet__, Point_generique):
                # pour garder la rfrence
                return self._Point_image(objet, self) 
            else:
                # dans, les autres cas, ce serait bien trop compliqu de garder la rfrence...
                return self.__call__(objet.__objet__)
        elif isinstance(objet, Point_generique):
            return self._Point_image(objet, self)
        elif isinstance(objet, Objet) and not isinstance(objet, (Objet_numerique, Transformation_generique)):
            return objet.__class__(**dict((key,  self(value)) for key, value in objet._iter_arguments))
        else:
            return objet





class Rotation(Transformation_generique):
    u"""Une rotation.

    Une rotation dfinie par son centre, et un angle en radian (r), grad (g) ou degr (d).
    L'unit par dfaut est le radian."""

    _Point_image = Point_rotation

    centre = __centre = Argument("Point_generique")
    angle = __angle = Argument("Angle_generique")
    unite = __unite = Argument("basestring")

    def __init__(self, centre = None, angle = math.pi, unite = "r", **styles):
        if centre is None:
            centre = Point(0, 0)
        if not isinstance(angle, ALL.Angle_generique):
            angle = ALL.Angle_libre(angle, unite)
        self.__centre = centre = Ref(centre)
        self.__angle = angle = Ref(angle)
        self.__unite = unite = Ref(unite)
        Transformation_generique.__init__(self, **styles)


    @property
    def radian(self):
        return self.__angle.radian
    
    rad = radian
    
    @property
    def degre(self):
        return self.__angle.degre
    
    deg = degre
    
    @property
    def grad(self):
        return self.__angle.grad

##    conversion = {"r": 1., "g": pi/200., "d": pi/180.}
##
##    @staticmethod
##    def choix_unite(unite):
##        u"Gestion des alias."
##        if unite is None:
##            return "r"
##        if unite == u"" or unite.startswith("d"):
##            return "d"
##        if unite.startswith("g"):
##            return "g"
##        return "r"


    def __call__(self, objet):
        if objet is self.__centre:
            return objet
        return Transformation_generique.__call__(self, objet)







class Translation(Transformation_generique):
    u"""Une translation.

    Une translation dfinie par un vecteur."""

    _Point_image = Point_translation

    vecteur = __vecteur = Argument("Vecteur_generique")

    def __init__(self, vecteur, **styles):
        if isinstance(vecteur, (tuple, list, types.ArrayType)):
            vecteur = ALL.Vecteur_libre(*vecteur)
        self.__vecteur = vecteur = Ref(vecteur)
        Transformation_generique.__init__(self, **styles)

    @staticmethod
    def _convertir(objet):
        if isinstance(objet, ALL.Vecteur_generique):
            return Translation(objet)
        raise TypeError, "%s must be of type 'Vecteur_generique'." %objet




class Reflexion(Transformation_generique):
    u"""Une symtrie axiale.

    Une symtrie axiale (rflexion) dfinie par une droite."""

    _Point_image = Point_reflexion

    droite = __droite = Argument("Ligne_generique")

    def __init__(self, droite, **styles):
        self.__droite = droite = Ref(droite)
        Transformation_generique.__init__(self, **styles)

    def __call__(self, objet):
        if objet is self.__droite:
            return objet
        return Transformation_generique.__call__(self, objet)




class Homothetie(Transformation_generique):
    u"""Une homothtie.

    Une homothtie dfinie par son centre et son rapport."""

    _Point_image = Point_homothetie

    centre = __centre = Argument("Point_generique")
    rapport = __rapport = Argument("Variable")

    def __init__(self, centre = None, rapport = 1, **styles):
        if centre is None:
            centre = Point(0, 0)
        self.__centre = centre = Ref(centre)
        self.__rapport = rapport = Ref(rapport)
        Transformation_generique.__init__(self, **styles)

    def __call__(self, objet):
        if objet is self.__centre:
            return objet
        return Transformation_generique.__call__(self, objet)





class Symetrie_centrale(Homothetie):
    u"""Une symtrie centrale.

    Une symtrie centrale dfinie par un point."""

    centre = __centre = Argument("Point_generique")

    def __init__(self, centre = None, **styles):
        self.__centre = centre = Ref(centre)
        Homothetie.__init__(self, centre, -1, **styles)



