#!/usr/bin/env python
# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##--------------------------------------#######
#   Mathlib 2 (sympy powered) #
##--------------------------------------#######
#WxGeometrie
#Dynamic geometry, graph plotter, and more for french mathematic teachers.
#Copyright (C) 2005-2007  Nicolas Pourcelot
#
#This program is free software; you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation; either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

## Objets complmentaires  ceux de sympy

import re, math

import sympy
#~ import end_user_functions
from sympy.printing.latex import LatexPrinter
from internal_objects import ObjetMathematique
import custom_functions
import mathlib
import pylib
import parsers


class Fonction(ObjetMathematique):
    def __init__(self, variables, expression):
        if not isinstance(variables, (list, tuple)):
            variables = (variables,)
        self.variables = variables
        self.expression = expression
    
    @classmethod
    def _substituer(cls, expression, dico):
        if hasattr(expression, "__iter__"):
            return expression.__class__(cls._substituer(elt, dico) for elt in expression)
        return expression.subs(dico)
    
    def __call__(self, *args, **kw):
        if kw:
            if args:
                raise ArgumentError, "les arguments sont entres de deux facons differentes."
            return self._substituer(self.expression, [(sympy.Symbol(key), value) for key, value in kw.iteritems()])
        if len(args) > len(self.variables):
            raise ArgumentError, "il y a plus d'arguments que de variables."
        return self._substituer(self.expression, zip(self.variables[:len(args)], args))
    
    def _variables(self):
        return tuple(str(arg) for arg in self.variables)
            
    def _expression(self, latex = False):
        if latex:
            expression = custom_functions.custom_latex(self.expression)
        else:
            expression = str(self.expression)
        return expression
    
    def __str__(self):
        #~ args = self.variables
        #~ expression = str(self.expression)
        #~ chaine = ""
        #~ for arg in args:
            #~ arg = str(arg)
            #~ expression = expression.replace(arg, arg.lstrip("_"))
            #~ chaine += ", " + arg[1:]
        #~ chaine += " -> " + expression
        return ", ".join(self._variables()) + " -> " + self._expression()
        
     
    for op in ("add", "mul", "div", "rdiv", "pow", "rpow"):    
        op = "__" + op + "__"
        def __op__(self, y, op = op):
            if isinstance(y, Fonction):
                if self.variables == y.variables or not y.variables:
                    return Fonction(self.variables, getattr(self.expression, op)(y.expression),)
                elif not self.variables:
                    return Fonction(y.variables, getattr(self.expression, op)(y.expression),)
                else:
                    raise ValueError, "les deux fonctions n'ont pas les memes variables."
            else:
                return Fonction(self.variables, getattr(self.expression, op)(y),)
        exec("%s=__op__" %op)
    del __op__
    
    def __ne__(self):
        return Fonction(self.variables, -self.expression)
        
    def __abs__(self):
        return Fonction(self.variables, abs(self.expression))

    def __eq__(self, y):
        if isinstance(y, Fonction):
            return self.expression == y.expression
        else:
            return self.expression == y
            
    def __gt__(self, y):
        if isinstance(y, Fonction):
            return self.expression > y.expression
        else:
            return self.expression > y




class ProduitEntiers(int):
    u"""Usage interne : destin  tre utilis avec sympy.factorint.
    
    Ajouter un attribut 'couples' APRS sa cration."""
    
    __slots__ = ["couples"]
    
        #~ def __init__(self, *couples):
            #~ self.couples = couples
            #~ n = 1
            #~ for entier, exposant in couples:
                #~ n *= entier**exposant
            #~ sympy.Integer.__init__(self, n)
    
    def __str__(self):
        def formater(exposant):
            if exposant == 1:
                return ""
            return "**" + str(exposant)
        return "*".join((str(entier) + formater(exposant)) for entier, exposant in self.couples)
        
##    @property
##    def is_postive(self):
##        return self > 0
##        
##    @property
##    def is_negative(self):
##        return self < 0
##        
##    @property
##    def is_nonpositive(self):
##        return self <= 0
##        
##    @property
##    def is_nonnegative(self):
##        return self >= 0
##        
##    @property
##    def is_zero(self):
##        return self == 0
##        
##    @property
##    def is_nonzero(self):
##        return self != 0


class CustomLatexPrinter(LatexPrinter):
    def _print_exp(self, expr, exp=None):
        tex = r"\mathrm{e}^{%s}" % self._print(expr.args[0])
        return self._do_exponent(tex, exp)
        
    def _print_Exp1(self, expr):
        return r"\mathrm{e}"
        
    def _print_ImaginaryUnit(self, expr):
        return r"\mathrm{i}"
        
    def _print_Fonction(self, expr):
        return ", ".join(expr._variables()) + "\\mapsto" + expr._expression(latex = True)[1:-1]
        
    def _print_Function(self, expr, exp=None):
        func = expr.func.__name__

        if hasattr(self, '_print_' + func):
            return getattr(self, '_print_' + func)(expr, exp)
            
        else:
            if exp is not None:
                name = r"\mathrm{%s}^{%s}" % (func, exp)
            else:
                name = r"\mathrm{%s}" % func
            if len(expr.args) == 1 and isinstance(expr.args[0], sympy.Symbol):
                return name + "(" + str(self._print(expr.args[0])) +")"
            else:
                args = [ str(self._print(arg)) for arg in expr.args ]
                return name + r"\left(%s\right)" % ",".join(args)

    def _print_ProduitEntiers(self, expr):
        def formater(exposant):
            if exposant == 1:
                return ""
            return "^" + str(exposant)
        return "\\times ".join((str(entier) + formater(exposant)) for entier, exposant in expr.couples)
            
    def _print_Real(self, expr):
        s = str(expr)
        if "e" in s:
            nombre,  exposant = s.split("e")
            return nombre + "\\times 10^{" + exposant.lstrip("+") + "}"            
        else:
            return s

    def _print_Infinity(self, expr):
        return r"+\infty"

    def _print_Order(self, expr):
        return r"\mathcal{O}\left(%s\right)" % \
            self._print(expr.args[0])

    def _print_abs(self, expr, exp=None):
        tex = r"\left|{%s}\right|" % self._print(expr.args[0])

        if exp is not None:
            return r"%s^{%s}" % (tex, exp)
        else:
            return tex

    def _print_Union(self, expr):
        tex = r"\cup".join(self._print(intervalle) for intervalle in expr.intervalles)
        tex = tex.replace(r"\}\cup\{", ";")
        return tex

    def _print_Intervalle(self, expr):
        if expr.inf_inclus and expr.sup_inclus and expr.inf == expr.sup:
            return r"\{%s\}" % self._print(expr.inf)
        if expr.inf_inclus:
            left = "["
        else:
            left = "]"
        if expr.sup_inclus:
            right = "]"
        else:
            right = "["
        return r"%s%s;%s%s" % (left, self._print(expr.inf), self._print(expr.sup), right)
        
    def _print_Vide(self, expr):
        return r"\varnothing"
        
    def _print_Singleton(self, expr):
        return r"\{%s\}" % self._print(expr.inf)
        
        
    
class LocalDict(dict):
    globals = {}
    def __getitem__(self, name):
        #~ print "Nom de clef: ", name
        #~ print "local: ", self.has_key(name)
        #~ print "global: ", self.globals.has_key(name)
        if self.has_key(name) or self.globals.has_key(name): # doit renvoyer une KeyError si la cl est dans le dictionnaire global, pour que Python y aille chercher ensuite la valeur associe  la cl
            return dict.__getitem__(self, name)
        return sympy.Symbol(name)

    def __setitem__(self, name, value):
        # Pour viter que l'utilisateur redfinisse pi, i, e, etc. par mgarde.
        if self.globals.has_key(name):
            raise KeyError, "%s est un nom reserve" %name
        if isinstance(value, str):
            # exec/eval encodent les chanes cres en utf8.
            value = value.decode("utf8").encode(param.encodage)
        dict.__setitem__(self, name, value)


     
            
class Interprete(object):
    def __init__(self, calcul_exact = True,
                                ecriture_scientifique = False, 
                                changer_separateurs = False, 
                                separateurs_personnels = (",", ";"), 
                                copie_automatique = False, 
                                formatage_OOo = True, 
                                formatage_LaTeX = False, 
                                ecriture_scientifique_decimales = 2, 
                                precision_calcul = 60, 
                                precision_affichage = 18,
                                simpify = True, 
                                verbose = None, 
                                ):
        # Dictionnaire local (qui contiendra toutes les variables dfinies par l'utilisateur).
        self.locals = LocalDict()
        # Dictionnaire global (qui contient les fonctions, variables et constantes prdfinies).
        self.globals = vars(mathlib.end_user_functions).copy()
        self.globals.update({
                "__builtins__": None,
                "__fonction__": Fonction,
                "__sympify__": sympy.sympify,
                "ans": self.ans,
                "__vars__": self.vars,
#                "__decimal__": Decimal,
                "__decimal__": self._decimal,
                "__type__": type, 
                "__local_dict__": self.locals,
                            })
        # pour viter que les procdures de rcriture des formules ne touchent au mots clefs,
        # on les rfrence comme fonctions (elles seront inaccessibles, mais ce n'est pas grave).
        # ainsi, "c and(a or b)" ne deviendra pas "c and*(a or b)" !
        self.globals.update({}.fromkeys(pylib.securite.__keywords_autorises__, lambda:None))

        self.locals.globals = self.globals
        
        self.calcul_exact = calcul_exact    # gerer les fractions et les racines de maniere exacte si possible.
        self.ecriture_scientifique = ecriture_scientifique # afficher les resultats en ecriture scientifique.
        self.changer_separateurs = changer_separateurs # appliquer les separateurs personnalises
        self.separateurs_personnels = separateurs_personnels # separateurs personnalises (separateur decimal, separateur de listes)
        # d'autres choix sont possibles, mais pas forcement heureux...
        self.copie_automatique = copie_automatique # copie automatique de chaque resultat dans le presse-papier
        self.formatage_OOo = formatage_OOo
        self.formatage_LaTeX = formatage_LaTeX
        self.ecriture_scientifique_decimales = ecriture_scientifique_decimales
        self.precision_calcul = precision_calcul
        self.precision_affichage = precision_affichage
        self.verbose = verbose
        self.simpify = simpify
        
        self.initialiser()
        
    def _decimal(self, nbr, prec = None):
        if prec is None:
            prec = self.precision_calcul
        return sympy.Real(nbr, prec)

    def initialiser(self):
        self.locals.clear()
        self.derniers_resultats = []

    def evaluer(self, calcul = ""):
        self.warning = ""
        # calcul = re.sub("[_]+", "_", calcul.strip()) # par mesure de scurit, les "__" sont interdits.
        # Cela permet ventuellement d'interdire l'accs  des fonctions.
        # Warning: inefficace. Cf. "_import _builtins_import _
        
        # Ferme automatiquement les parentheses.
        parentheses = "({[", ")}]"
        for i in range(3):
            difference = calcul.count(parentheses[0][i])-calcul.count(parentheses[1][i])
            if difference > 0:
                calcul += difference*parentheses[1][i]
                self.warning += u" Attention, il manque des parenthses \"" + parentheses[1][i] + "\"."
            elif difference < 0:
                self.warning += u" Attention, il y a des parenthses \"" + parentheses[1][i] + "\" superflues."
                if calcul.endswith(abs(difference)*parentheses[1][i]):
                    calcul = calcul[:difference]
 
        if self.verbose:
            print "Traitement ({[]}) :  ", calcul

        if calcul[0] in "><!=^*/%+":
            calcul = "_" + calcul

        if self.verbose:
            print "Traitement ><!=^*/%+ :  ", calcul
            
        if ">>" in calcul:
            liste = calcul.split(">>")
            calcul = liste[0]
            for s in liste[1:]:
                calcul = s + "(" + calcul + ")"
        
        if self.verbose:
            print "Traitement >> :  ", calcul

        # Cas d'une fonction.
        # Exemple: 'f(x,y)=x+y+3' sera traduit en 'f=__fonction__((x,y), x+y+3)'
        if re.match("[^=()]+[(][^=()]+[)][ ]*=[^=]", calcul):
            var, val = calcul.split("=", 1)
            var = var.strip()
            i = var.find("(")
            nom = var[:i]
            variables = var[i:]
            calcul = nom + "=__fonction__(" + variables + "," + self._traduire(val) + ")"
            self._executer(calcul)
        else:
            self._executer(self._traduire(calcul))
        
        self.derniers_resultats.append(self.locals["_"])
        
        if not self.calcul_exact and hasattr(self.locals["_"], "evalf"):
            return self._formater(self.locals["_"].evalf(self.precision_calcul))
        return self._formater(self.locals["_"])


    def _ecriture_scientifique(self, chaine):
        valeur = float(chaine)
        mantisse = int(math.floor(math.log10(abs(valeur))))
        chaine = str(round(valeur/10.**mantisse, self.ecriture_scientifique_decimales))
        if mantisse:
            chaine += "*10^"+str(mantisse)
        return  chaine

    def _ecriture_scientifique_latex(self, chaine):
        valeur = float(chaine)
        mantisse = int(math.floor(math.log10(abs(valeur))))
        chaine = str(round(valeur/10.**mantisse, self.ecriture_scientifique_decimales))
        if mantisse:
            chaine += "\\times 10^{%s}" %mantisse
        return  chaine


##    def _ecriture_scientifique_latex(self, valeur):
##        mantisse = int(math.floor(math.log10(abs(valeur))))
##        latex = resultat = str(round(valeur/10.**mantisse, self.ecriture_scientifique_decimales))
##        if mantisse:
##            resultat += "*10^"+str(mantisse)
##            latex += "\\times 10^{%s}" %mantisse
##        return  resultat, "$" + latex + "$"


    def _formater_decimaux(self, chaine):
        if "." in chaine:
##            print "r1", chaine, self.precision_calcul, self.precision_affichage, sympy.Real(str(chaine), self.precision_calcul)
            chaine = str(sympy.Real(str(chaine), self.precision_calcul).evalf(self.precision_affichage))
##            print "r2", chaine
            chaine = chaine.rstrip('0')
##            print "r3", chaine
            if chaine.endswith("."):
                chaine = chaine[:-1]
        return chaine

 
    def _formatage_simple(self, valeur):
        resultat = pylib.unicode2(valeur)        
        # log -> ln
        resultat = pylib.regsub("(?<![A-Za-z0-9_])log[(]", resultat, "ln(")
        # E -> e
        resultat = pylib.regsub("(?<![A-Za-z0-9_.])E(?![A-Za-z0-9_])", resultat, "e")
        # I -> i
        resultat = pylib.regsub("(?<![A-Za-z0-9_])I(?![A-Za-z0-9_])", resultat, "i")
        # 452.25e+54 -> 452.25*10^54
        def puiss10(s):
            return s[:-2] + "*10^" + (s[-1] == "-" and "-" or "")
        resultat = pylib.regsub("(?<![A-Za-z0-9._])[0-9.]+e[-+]", resultat, puiss10)
        # ** -> ^
        resultat = resultat.replace("**", "^")
##        #~ resultat = resultat.replace("-", " - ").replace("+", " + ").replace("e - ", "E-").replace("e + ", "E+")
        # -x-25 -> -x - 25
        resultat = pylib.regsub("(?<=[A-Za-z0-9._])[-+](?=[A-Za-z0-9._])", resultat, lambda s: " " + s + " ")
##        .replace("+", " + ") = resultat.replace("-", " - ").replace("+", " + ").replace("- >", "->")
        return resultat
   
    def _formater(self, valeur):
##        if isinstance(valeur, sympy.Real):
##            valeur = valeur.evalf(self.precision_affichage)
##        if not self.calcul_exact and self.ecriture_scientifique and isinstance(valeur, sympy.Number):
##            resultat, latex = self._ecriture_scientifique(valeur)
##        else:
        resultat = self._formatage_simple(valeur)
        if valeur is None:
            latex = ""
        else:
            try:
                latex = mathlib.custom_functions.custom_latex(valeur)
            except Exception:
                pylib.print_error()
                latex = "<Affichage impossible>"
        
        
            
        if self.ecriture_scientifique and not self.calcul_exact:
            resultat = pylib.regsub(parsers.NBR, resultat, self._ecriture_scientifique)
            latex = pylib.regsub(parsers.NBR, latex, self._ecriture_scientifique_latex)
        else:
##            print "initial", resultat
            resultat = pylib.regsub(parsers.NBR, resultat, self._formater_decimaux)
##            print "final", resultat
            latex = pylib.regsub(parsers.NBR, latex, self._formater_decimaux)
            
##        if re.match("[0-9]*[.][0-9]+$", resultat):
##            resultat = resultat.rstrip('0')
##            if resultat.endswith("."):
##                resultat += "0"
##            latex = "$" + resultat + "$"
        if self.changer_separateurs:
            resultat = resultat.replace(",", self.separateurs_personnels[1]).replace(".", self.separateurs_personnels[0])
            latex = latex.replace(",", self.separateurs_personnels[1]).replace(".", self.separateurs_personnels[0])
        
        if resultat == "oo":
            resultat = "+oo"
        
        return resultat, latex


    def _traduire(self, formule):
        variables = self.globals.copy()
        variables.update(self.locals)
        #callable_types = (  types.BuiltinFunctionType,
                            #types.BuiltinMethodType, 
                            #types.FunctionType, 
                            #types.UfuncType, 
                            #types.MethodType, 
                            #types.TypeType, 
                            #types.ClassType)
        #fonctions = [key for key, val in variables.items() if isinstance(val, callable_types)]
        fonctions = [key for key, val in variables.items() if hasattr(val, "__call__")]
        #print fonctions
        # La fonction traduire_formule de la librairie mathlib.formatage permet d'effectuer un certain nombre de conversions.
        formule = parsers.traduire_formule(formule, fonctions, 
                        OOo = self.formatage_OOo, 
                        LaTeX = self.formatage_LaTeX, 
                        changer_separateurs = self.changer_separateurs, 
                        separateurs_personnels = self.separateurs_personnels, 
                        simpify = self.simpify, 
                        verbose = self.verbose, 
                        )
        i = formule.find("resoudre(")
        if i != -1:
            formule = formule.replace("*ou*", " ou ").replace("*et*", " et ")
            deb, bloc, fin = pylib.split_around_parenthesis(formule, i)
            formule = deb + '("' + bloc[1:-1] + '", local_dict = __local_dict__)'
        if self.verbose is not False:
            print "test_resoudre: ", i, formule
        return formule


    def _executer(self, instruction):
        # dans certains cas, il ne faut pas affecter le rsultat  la variable "_" (cela provoquerait une erreur de syntaxe)
        # (Mots cls devant se trouver en dbut de ligne : dans ce cas, on ne modifie pas la ligne)
##        if True in [instruction.startswith(exception + " ") for exception in securite.__keywords_debut_ligne__]:
        def re_keywords(liste):
            global pylib
            return "("+"|".join(pylib.securite.__keywords_debut_ligne__)+")[ :(;'\"]"

        if re.match(re_keywords(pylib.securite.__keywords_debut_ligne__), instruction):
            self.locals["_"] = None
        else:
            instruction = "_=" + instruction
        
##        keywords_interdits = [kw + " " for kw in securite.__keywords_interdits__] + [kw + "(" for kw in securite.__keywords_interdits__]
##        instruction = recursive_mreplace(instruction, keywords_interdits)
        pylib.regsub(re_keywords(pylib.securite.__keywords_interdits__), instruction, " ")

        #print self.globals.keys()
        exec(instruction, self.globals, self.locals)
        
    
    def vars(self):
        dictionnaire = self.globals.copy()
        dictionnaire.update(self.locals)
        return dictionnaire


    def ans(self, n = -1):
        n = int(n)
        if self.derniers_resultats: return self.derniers_resultats[n]
        self.warning += u" Ans(): aucun calcul antrieur." 
        return 0

