# -*- coding: iso-8859-1 -*-
from __future__ import division # 1/2 == .5 (par defaut, 1/2 == 0)

##########################################################################
#
#       Fonctions couramment utilisees, et non implementees en Python
#                           (ou non connues de moi !)
#
##########################################################################
#    WxGeometrie
#    Dynamic geometry, graph plotter, and more for french mathematic teachers.
#    Copyright (C) 2005-2007  Nicolas Pourcelot
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

# version unicode

import param # paramtres du programme
import re   
import weakref
import sys
import os.path
import warnings
try:
    import wx
except:
    pass
    
def iff(condition, value1, value2 = ""):
    u"""Renvoie "value1" si "condition" est verifi, "value2" sinon (par defaut, '')."""
    if condition: return value1
    return value2
    
def is_in(element, _list):
    u"""Teste si l'lement est dans la liste, en effectuant un test d'identit (is) et non d'galit (==)."""
    test = False
    for elt in _list:
        if elt is element:
            test = True
            break
    return test

# This is a lot slower (2.4 times about) :
##def isin2(element, _list):
##    u"""Teste si l'lement est dans la liste, en effectuant un test d'identit (is) et non d'galit (==)."""
##    return id(element) in (id(elt) for elt in _list)

# And this too... (2 times about on python 2.5)
##def isin3(element, _list):
##    u"""Teste si l'lement est dans la liste, en effectuant un test d'identit (is) et non d'galit (==)."""
##    return any(elt is element for elt in _list)

def mreplace(main_string, list_of_strings, new_string = ""):
    u"""Remplace, dans "main_string", toutes les sous-chaines de "list_of_strings", par la chaine "new_string"."""
    for old_string in list_of_strings:
        main_string = main_string.replace(old_string, new_string)
    return main_string

def recursive_replace(main_string, old_string, new_string = "", max_loops = 10000, max_len = 1000000):
    u"""Remplace, dans "main_string", la sous-chane "old_string" par "new_string", au besoin en plusieurs passes.
    
    En fin de processus, la sous-chane old_string ne subsiste plus dans la chane.
    Renvoie une erreur si le processus ne semble pas converger.
    (C'est en particulier le cas si old_string est strictement inclus dans new_string)
    
    La diffrence avec replace est claire sur cette exemple :
    >>> "Hi HelloHello world world !".replace("Hello world", "")
    'Hi Hello world !'
    >>> recursive_replace("Hi HelloHello world world !", "Hello world", "")
    'Hi  !'
    
    Il y a un cas particulier ou la sous-chane reste prsente :
    >>> recursive_replace("Hi HelloHello world world !", "Hello world", "Hello world")
    'Hi HelloHello world world !'   
    """
    
    chaine = ""
    loops = 0
    while chaine <> main_string:
        loops += 1
        chaine = main_string
        main_string = main_string.replace(old_string, new_string)
        if loops > max_loops:
            raise RuntimeError, "Nombre de passes superieur au maximum autorise."
        if len(main_string) > max_len:
            raise OverflowError, "Taille de la chaine superieure au maximum autorise."
    return main_string


def recursive_mreplace(main_string, list_of_strings, new_string = "", max_loops = 10000, max_len = 1000000):
    u"""Remplace, dans "main_string", toutes les sous-chaines de "list_of_strings" par "new_string", au besoin en plusieurs passes.
    
    En fin de processus, la sous-chane old_string ne subsiste plus dans la chane.
    Renvoie une erreur si le processus ne semble pas converger.
    
    Voir galement recursive_replace() et mreplace().
    
    Remarque: recursive_mreplace n'est pas quivalent des appels successifs de recursive_replace().
    >>> s="tbtbaoao"
    >>> s = recursive_mreplace(s, ("to", "ba"))
    >>> s
    ''
    >>> s="tbtbaoao"
    >>> for i in ("to", "ba"):s = recursive_replace(s, i)
    >>> s
    'tbtoao'    
    """
    
    chaine = ""
    loops = 0
    while chaine <> main_string:
        loops += 1
        chaine = main_string
        main_string = mreplace(main_string, list_of_strings, new_string)
        if loops > max_loops:
            raise RuntimeError, "Nombre de passes superieur au maximum autorise."
        if len(main_string) > max_len:
            raise OverflowError, "Taille de la chaine superieure au maximum autorise."
    return main_string


def mfind(chaine, car):
    n = 0
    l = []
    while True:
        n = chaine.find(car, n)
        if n == -1: break
        l.append(n)
        n += 1
    return l
    
    

def msplit(main_string, list_of_separators):
    u"""Dcoupe la chaine "main_string", selon les sparateurs dfinis dans "list_of_separators"."""
    return mreplace(main_string, list_of_separators[1:], list_of_separators[0]).split(list_of_separators[0])



def removeend(main_string, *substrings):
    u"Enlve les ventuelles occurences de substring en fin de chaine."
    if substrings and True not in (sub == "" for sub in substrings): # pour viter une ventuelle boucle infinie.
        test = True
        while test:
            for sub in substrings:
                if main_string.endswith(sub):
                    main_string =  main_string[:-len(sub)]
                    test = None
            if test is None:
                test = True
            else:
                test = False
    return main_string


def removestart(main_string, *substrings):
    u"Enlve les ventuelles occurences de substring en dbut de chaine."
    if substrings and True not in (sub == "" for sub in substrings): # pour viter une ventuelle boucle infinie.
        test = True
        while test:
            for sub in substrings:
                if main_string.startswith(sub):
                    main_string =  main_string[len(sub):]
                    test = None
            if test is None:
                test = True
            else:
                test = False
    return main_string


def no_twin(liste):
    u"""Elimine les doublons dans une liste.
    Si tous les lements de la liste sont 'hashables', mieux vaut utiliser la fonction set."""
    dico = {}
    for elt in liste:dico[id(elt)] = elt
    return dico.values()
    
#def ntwin(l): return dict((id(elt), elt) for elt in l).values() # plus lgant, mais 50% plus lent ?!?

#~ def advanced_split_old(main_string, separator): # DESUET
    #~ u"""Dcoupe la chaine "main_string" de manire intelligente, 
    #~ en ignorant les sparateurs compris dans un groupe entre parenthses, crochets ou accolades."""
    #~ lieux = mfind(main_string, separator)
    #~ liste = []
    #~ old = 0
    #~ for lieu in lieux:  # on ne conserve que les "bonnes" separations.
        #~ ecarts = 0
        #~ parentheses = "({[", ")}]"
        #~ for i in range(3):
            #~ ecarts += abs(main_string.count(parentheses[0][i], 0, lieu)-main_string.count(parentheses[1][i], 0, lieu))
        #~ if not ecarts:
            #~ liste.append(main_string[old:lieu])
            #~ old = lieu + 1
    #~ return liste + [main_string[old:]]


    
def advanced_split(main_string, separator, keep_empty_str = False, symbols = "([{}])"):
    u"""Dcoupe la chaine "main_string" de manire intelligente, 
    en ignorant les sparateurs compris dans un groupe entre parenthses, crochets, accolades, guillemets.
    Attention, separateur ne peut donc pas tre une parenthse, un crochet, une accolade ou un guillemet !
    Par dfaut, supprime galement les chaines vides."""
    in_string = False # est-on dans une chaine ?
    in_string_sep = "'" # caractere encadrant la chaine (" ou ')
    parentheses = 0 # tient le compte des parentheses ouvertes non fermees
    crochets = 0 # idem pour les crochets
    accolades = 0 # idem
    coupures = [-1] # endroits ou il faudra couper la chaine
    for i in xrange(len(main_string)):
        a = main_string[i]
        if a in ("'", '"'):
            if in_string:
                if in_string_sep == a: # attention, il y a 2 indicateurs de chaine (" et ')
                    in_string = False
            else:
                in_string = True
                in_string_sep = a
        elif a in symbols:
            if a == "(" and not in_string:
                parentheses += 1
            elif a == ")" and not in_string:
                parentheses -= 1
            elif a == "[" and not in_string:
                crochets += 1
            elif a == "]" and not in_string:
                crochets -= 1
            elif a == "{" and not in_string:
                accolades += 1
            elif a == "}" and not in_string:
                accolades -= 1
        elif a == separator and not (in_string or parentheses or crochets or accolades) :
            coupures.append(i)
    coupures.append(None)
    # chaine[i:None] retourne la fin de chaine
    
    return [main_string[i+1:j] for i, j in zip(coupures[:-1], coupures[1:]) if main_string[i+1:j] or keep_empty_str]




def regexp(regular_exp, main_string, action = "''"): # DESUET, utiliser dsormais regsub
    u"""Transforme la chaine "main_string" :
    Il applique aux parties vrifiant "regular_exp" le traitement "action".
    Le traitement "action" doit avoir pour variable "x",
    x correspondant a un bout de chaine vrifiant "regular_exp".
    Par exemple, action peut tre "x[:-1] + '*' + x[-1]"."""
    deprecation("Use regsub instead of regexp.")
    l = re.split("(" + regular_exp + ")", main_string)
    exec("def f(x): return " + (action or "''"))
    for i in range(len(l)//2):
        l[2*i+1] = f(l[2*i+1])
    return "".join(l)
    
    
def regsub(regular_exp, main_string, action = ""):
    u"""Transforme la chaine "main_string" :
    Il applique aux parties vrifiant "regular_exp" le traitement "action".
    
    >>> regsub("[a-z]", "salut les amis !", "?")
    '????? ??? ???? !'
    >>> regsub("[a-z]+", "hello world !", lambda s: s[1:])
    'ello orld !'
    """
    if isinstance(action, basestring):
        return re.sub(regular_exp, action, main_string)
    else:
        return re.sub(regular_exp, lambda x: action(x.group(0)), main_string)



    
    
class WeakList(weakref.WeakValueDictionary):
    u"""Une 'liste' de rferences faibles.
    
    Le terme 'liste' est trompeur, la syntaxe des listes de python n'est pas implmente,
    excepte les mthodes append(), et remove(), et la conversion en liste.
    
    En outre, le contenu s'obtient par la mthode values().
    
    Note:
    L'implmentation de remove est un peu diffrente :
    'remove' utilise le comparateur 'is', et non "==", ce qui fait que remove([]) ne fera jamais rien, par exemple.
    (Il faut qu'il s'agisse du meme objet, et non d'un objet gal).
    Sinon, il faut utiliser compare_and_remove."""
    
    def __init__(self):
        weakref.WeakValueDictionary.__init__(self)
        
    def append(self, valeur):
        u"Ajoute une valeur en fin de liste."
        self[max(self.keys() or [0]) + 1] = valeur
        
    def remove(self, valeur):
        u"""Supprime la valeur de la liste.
        
        Un test d'identit (et non d'galit) est effectu ('is' et non '==').
        Si la valeur est prsente plusieurs fois, elle n'est supprime qu'une seule fois.
        Si la valeur n'est pas prsente, une erreur de type ValueError est mise.
        """
        for key, value in self.items():
            if value is valeur:
                del self[key] # il faut qu'il s'agisse du mme objet
                return
        raise ValueError,  repr(valeur) + " is not in WeakList"
            
    def compare_and_remove(self, valeur):
        u"""Supprime la valeur de la liste.
        
        Un test d'galit est effectu ('==' et non 'is').
        Si la valeur est prsente plusieurs fois, elle n'est supprime qu'une seule fois.
        Si la valeur n'est pas prsente, une erreur de type ValueError est mise.
        """
        for key, value in self.items():
            if value == valeur:
                del self[key] # un objet gal suffit
                return
        raise ValueError,  repr(valeur) + " not in WeakList"

    def remove_all(self, valeur):
        u"""Supprime la valeur de la liste.
        
        Un test d'identit (et non d'galit) est effectu ('is' et non '==').
        Toutes les occurences de la valeur sont supprimes.
        Si la valeur n'est pas prsente, aucune erreur n'est mise."""
        for key, value in self.items():
            if value is valeur: del self[key] # il faut qu'il s'agisse du mme objet
            
    def compare_and_remove_all(self, valeur):
        u"""Supprime la valeur de la liste.
        
        Un test d'galit est effectu ('==' et non 'is').
        Toutes les occurences de la valeur sont supprimes.
        Si la valeur n'est pas prsente, aucune erreur n'est mise."""
        for key, value in self.items():
            if value == valeur: del self[key] # un objet gal suffit

    def __str__(self):
        return str(self.values()) + " (WeakList)"
        
    def __iter__(self):
        return self.itervalues()
            
    def __getitem__(self, n):
        return self.values()[n]



##class WeakSet:
##    u"""Une classe 'set' utilisant des rfrences faibles.
##    
##    Backported from Python 3000b3. Copyright (c) 2001-2008 Python Software Foundation."""
##    def __init__(self, data=None):
##        self.data = set()
##        def _remove(item, selfref=ref(self)):
##            self = selfref()
##            if self is not None:
##                self.data.discard(item)
##        self._remove = _remove
##        if data is not None:
##            self.update(data)
##
##    def __iter__(self):
##        for itemref in self.data:
##            item = itemref()
##            if item is not None:
##                yield item
##
##    def __len__(self):
##        return sum(x() is not None for x in self.data)
##
##    def __contains__(self, item):
##        return ref(item) in self.data
##
##    def __reduce__(self):
##        return (self.__class__, (list(self),),
##                getattr(self, '__dict__', None))
##
##    def add(self, item):
##        self.data.add(ref(item, self._remove))
##
##    def clear(self):
##        self.data.clear()
##
##    def copy(self):
##        return self.__class__(self)
##
##    def pop(self):
##        while True:
##            try:
##                itemref = self.data.pop()
##            except KeyError:
##                raise KeyError('pop from empty WeakSet')
##            item = itemref()
##            if item is not None:
##                return item
##
##    def remove(self, item):
##        self.data.remove(ref(item))
##
##    def discard(self, item):
##        self.data.discard(ref(item))
##
##    def update(self, other):
##        if isinstance(other, self.__class__):
##            self.data.update(other.data)
##        else:
##            for element in other:
##                self.add(element)
##    def __ior__(self, other):
##        self.update(other)
##        return self
##
##    # Helper functions for simple delegating methods.
##    def _apply(self, other, method):
##        if not isinstance(other, self.__class__):
##            other = self.__class__(other)
##        newdata = method(other.data)
##        newset = self.__class__()
##        newset.data = newdata
##        return newset
##
##    def difference(self, other):
##        return self._apply(other, self.data.difference)
##    __sub__ = difference
##
##    def difference_update(self, other):
##        if self is other:
##            self.data.clear()
##        else:
##            self.data.difference_update(ref(item) for item in other)
##    def __isub__(self, other):
##        if self is other:
##            self.data.clear()
##        else:
##            self.data.difference_update(ref(item) for item in other)
##        return self
##
##    def intersection(self, other):
##        return self._apply(other, self.data.intersection)
##    __and__ = intersection
##
##    def intersection_update(self, other):
##        self.data.intersection_update(ref(item) for item in other)
##    def __iand__(self, other):
##        self.data.intersection_update(ref(item) for item in other)
##        return self
##
##    def issubset(self, other):
##        return self.data.issubset(ref(item) for item in other)
##    __lt__ = issubset
##
##    def __le__(self, other):
##        return self.data <= set(ref(item) for item in other)
##
##    def issuperset(self, other):
##        return self.data.issuperset(ref(item) for item in other)
##    __gt__ = issuperset
##
##    def __ge__(self, other):
##        return self.data >= set(ref(item) for item in other)
##
##    def __eq__(self, other):
##        return self.data == set(ref(item) for item in other)
##
##    def symmetric_difference(self, other):
##        return self._apply(other, self.data.symmetric_difference)
##    __xor__ = symmetric_difference
##
##    def symmetric_difference_update(self, other):
##        if self is other:
##            self.data.clear()
##        else:
##            self.data.symmetric_difference_update(ref(item) for item in other)
##    def __ixor__(self, other):
##        if self is other:
##            self.data.clear()
##        else:
##            self.data.symmetric_difference_update(ref(item) for item in other)
##        return self
##
##    def union(self, other):
##        return self._apply(other, self.data.union)
##    __or__ = union
##
##    def isdisjoint(self, other):
##        return len(self.intersection(other)) == 0


# Quasi un alias de print_exc() que je connaissais pas... ;)
def print_error(chaine = ""):
    u"Affiche l'erreur sans interrompre le programme."
    if param.debug:
        if chaine:
            print(chaine)
        sys.excepthook(*sys.exc_info())
        print ("Warning: this error was not raised.")
    
# traceback.format_exc existe dj !
"""def return_error():
    "Retourne sous forme de chane l'erreur, sans interrompre le programme."
    typ, val, tb = sys.exc_info()
    tb = traceback.extract_tb(tb)
    return 'Traceback (most recent call last):\n' + str(tb) + '\n' + str(typ) + ": " + str(val) + '\n'"""

def print_error(chaine = ""):  # TEMPORAIRE (contourne un bug de wxpython)
    u"Affiche l'erreur sans interrompre le programme."
    if param.debug:
        if chaine:
            print(chaine)
        import traceback
        typ, val, tb = sys.exc_info()
        tb = traceback.extract_tb(tb)
        print 'Traceback (most recent call last)'
        for fichier, ligne, fonction, code in tb:
            print '    File "' + unicode2(fichier) +'", line ' + str(ligne) + ', in ' + unicode2(fonction)
            print '        ' + unicode2(code, "iso-8859-1")
        print unicode2(typ.__name__) + ": " + unicode2(val)
        print "Warning: this error was not raised."


def split_geoname(name):
    u"""Tente de dcomposer un nom d'objet gomtrique en plusieurs noms.
    
    Ex:
    1) "AB" -> ("A","B")
    2) "A12B" -> ("A12","B")
    3) "AB1" -> ("A","B1")
    4) "A'B\"" -> ("A'", "B\"")
    5) "ABC" -> ("A", "B", "C")
    """
    
    return tuple(nom.strip() for nom in re.split("""([ ]*[A-Za-z][_]?[0-9"']*[ ]*)""", name) if nom)
    
    
def convert_geoname(name, level = 0):
    u"""Convertit le nom entr par l'utilisateur en un nom rellement interprtable.
    
    Une conversion de niveau 1 est applique dans les botes de dialogue.
    
    Une conversion de niveau 0 est applique dans la console."""
    
    if level > 0:
        if level > 1:
            if " " not in name:
                name = " ".join(split_geoname)
        name = name.replace('"', "''")
        name = name.replace("'''", "_tierce")
        name = name.replace("''", "_seconde")
        name = name.replace("'", "_prime")
    name = name.replace("```", "_tierce")
    name = name.replace("``", "_seconde")
    name = name.replace("`", "_prime")
    
    
    
    
def split_around_parenthesis(main_string, position = 0, leftbracket = "("):
    u"""Coupe le premier groupe entre parentheses rencontr, en tenant compte des guillemets.
    
    'leftbracket' peut prendre les valeurs "(", "[" ou "{"
    La parenthese ouvrante du groupe sera la premire trouve  droite de 'position' 
    Exemple: '1+4*(5+3*(2+7)+2-")")*7+(2-3)+4' -> ['1+4*', '(5+3*(2+7)+2-")")', '*7+(2-3)+4']
    """
    in_string = False # est-on dans une chaine ?
    in_string_sep = "'" # caractere encadrant la chaine (" ou ')
    position = main_string.find(leftbracket, position)
    if position is -1:
        return (main_string,)
    parentheses = 1 # tient le compte des parentheses ouvertes non fermees
    rightbracket = {"(": ")", "[": "]", "{": "}"}[leftbracket]
    prefixe = main_string[:position]
    chaine = main_string[position + 1:]
    for i in xrange(len(chaine)):
        a = chaine[i]
        if a in ("'", '"'):
            if in_string:
                if in_string_sep == a: # attention, il y a 2 indicateurs de chaine (" et ')
                    in_string = False
            else:
                in_string = True
                in_string_sep = a
        elif a == leftbracket and not in_string:
            parentheses += 1
        elif a == rightbracket and not in_string:
            parentheses -= 1
            if parentheses is 0:
                chaine = chaine
                return (prefixe, leftbracket + chaine[:i + 1], chaine[i + 1:])
    return (main_string,) # aucune parenthese fermante n'a t trouve pour ce groupe.

def warning(message, type_warning = Warning):
    if param.warning:
        warnings.warn(message, type_warning, stacklevel = 3)
    
def deprecation(message):
    warnings.warn(message, DeprecationWarning, stacklevel = 3)

def unicode2(string_or_unicode, encodage = None):
    u"Convertit en unicode si besoin est, avec l'encodage de 'param.encodage' par dfaut."
    if isinstance(string_or_unicode, str):
        try:
            return unicode(string_or_unicode, encodage or param.encodage)
        except UnicodeDecodeError:
#            try:
#                print "chaine :\n", string_or_unicode
#                print unicode(string_or_unicode, "cp1252")
#            except Exception:
#                pass
            raise
    elif isinstance(string_or_unicode, unicode):
        return string_or_unicode
    else:
        return unicode(string_or_unicode)

def str2(string_or_unicode, encodage = None):
    u"Convertit en string si besoin est, avec l'encodage de 'param.encodage' par dfaut."
    if isinstance(string_or_unicode, str):
        return string_or_unicode
    elif isinstance(string_or_unicode, unicode):
        return string_or_unicode.encode(encodage or param.encodage)
    else:
        return str(string_or_unicode)

    
##def emplacement(*liste): # DESUET
##    u"""Emplacement relatif au lieu d'excution du programme.
##    
##    Exemple: emplacement("images", "archives", "old.png").
##    
##    Mise  jour : Utiliser plutt dsormais path2("%/images/archives/old.png")."""
##    deprecation("Use path2 instead of emplacement.")
##    return os.path.join(unicode2(param.EMPLACEMENT), *liste)
    
    
def png(nom):
    u"""Charge l'image <nom>.png depuis le repertoire 'images/'.
    
    L'image doit tre au format png, et son nom doit indiqu sans l'extension '.png'."""
    return wx.Image(os.path.join(unicode2(param.EMPLACEMENT), 'images', nom + ".png"), wx.BITMAP_TYPE_PNG).ConvertToBitmap()
    
    
def path2(chemin):
    u"""Transforme le chemin en remplaant les / et \\ selon le sparateur utilis par le systme.
    
    % est remplac par l'emplacement du programme (contenu dans param.EMPLACEMENT).
    Exemple : path2("%/images/archives/old.png").
    ~ fait rfrence au rpertoire personnel de l'utilisateur (ex: /home/SteveB/ sous Linux.
    """
    return os.path.normpath(os.path.expanduser(unicode2(chemin).replace("%", unicode2(param.EMPLACEMENT))))
    

if sys.version_info<(2,5):
    def all(iterable):
        '''all(iterable) -> bool
        
        Return True if bool(x) is True for all values x in the iterable.'''
        for element in iterable:
            if not element:
                return False
        return True
        #~ False not in (bool(x) for x in iterable)
        
    def any(iterable):
        '''any(iterable) -> bool
        
        Return True if bool(x) is True for any x in the iterable.'''
        for element in iterable:
            if element:
                return True
        return False
        #~ True in (bool(x) for x in iterable)

# L'ide de compiler en une fois pour toute les expressions regulires n'est pas avantageuse : le temps gagn ainsi est perdu  rechercher les entres dans le dictionnaire.

#~ def regsub(regular_exp, main_string, action = ""):
    #~ u"""Transforme la chaine "main_string" :
    #~ Il applique aux parties vrifiant "regular_exp" le traitement "action".
    
    #~ >>> regsub("[a-z]", "salut les amis !", "?")
    #~ '????? ??? ???? !'
    #~ >>> regsub("[a-z]+", "hello world !", lambda s: s[1:])
    #~ 'ello orld !'
    #~ """
    #~ if isinstance(regular_exp, basestring):
        #~ if isinstance(action, basestring):
            #~ return re.sub(regular_exp, action, main_string)
        #~ else:
            #~ return re.sub(regular_exp, lambda x: action(x.group(0)), main_string)
    #~ else:
        #~ if isinstance(action, basestring):
            #~ return regular_exp.sub(action, main_string)
        #~ else:
            #~ return regular_exp.sub(lambda x: action(x.group(0)), main_string)


#~ class REStorageDict(dict):
    #~ u"""Un dictionnaire qui stocke les RE sous forme compile.
    
    #~ """
    #~ def __getitem__(self, name):
        #~ try:
            #~ return dict.__getitem__(self, name)
        #~ except KeyError:
            #~ value = re.compile(name)
            #~ self.__setitem__(name, value)
            #~ return value


class WeakRef(weakref.ref):
    u"""WeakRef surclasse weakref.ref en modifiant sa mthode '__eq__'.
    
    a == b <=> type(a) == type(b) == WeakRef and a() is b().
    Le but est de ne pas appeler les mthodes __eq__ des objets rfrencs."""
    
    def __eq__(self, y):
        if not (isinstance(self, WeakRef) and isinstance(y, WeakRef)):
            return False
        if self() is None or y() is None:
            return self is y
        return  self() is y()
        


class CustomWeakKeyDictionary(weakref.WeakKeyDictionary):
    """WeakKeyDictionary utilisant Weakref au lieu de weakref.ref.
    """

    def __init__(self, dict=None):
        self.data = {}
        def remove(k, selfref=weakref.ref(self)):
            self = selfref()
            if self is not None:
                del self.data[k]
        self._remove = remove
        if dict is not None: self.update(dict)

    def __delitem__(self, key):
        del self.data[WeakRef(key)]

    def __getitem__(self, key):
        return self.data[WeakRef(key)]

    def __repr__(self):
        return "<WeakKeyDictionary at %s>" % id(self)

    def __setitem__(self, key, value):
        self.data[WeakRef(key, self._remove)] = value

    def copy(self):
        new = CustomWeakKeyDictionary()
        for key, value in self.data.items():
            o = key()
            if o is not None:
                new[o] = value
        return new

    def get(self, key, default=None):
        return self.data.get(WeakRef(key),default)

    def has_key(self, key):
        try:
            wr = WeakRef(key)
        except TypeError:
            return 0
        return wr in self.data

    def __contains__(self, key):
        try:
            wr = WeakRef(key)
        except TypeError:
            return 0
        return wr in self.data



    def pop(self, key, *args):
        return self.data.pop(WeakRef(key), *args)

    def setdefault(self, key, default=None):
        return self.data.setdefault(WeakRef(key, self._remove),default)

    def update(self, dict=None, **kwargs):
        d = self.data
        if dict is not None:
            if not hasattr(dict, "items"):
                dict = type({})(dict)
            for key, value in dict.items():
                d[WeakRef(key, self._remove)] = value
        if len(kwargs):
            self.update(kwargs)
